import textwrap

import pytest

from salt.modules import devinfo
from tests.support.mock import patch


@pytest.fixture
def configure_loader_modules():
    return {devinfo: {}}


def test_udev():
    assert devinfo._udev({"A": {"B": 1}}, "a.b") == 1
    assert devinfo._udev({"A": {"B": 1}}, "A.B") == 1
    assert devinfo._udev({"A": {"B": 1}}, "a.c") == "n/a"
    assert devinfo._udev({"A": [1, 2]}, "a.b") == "n/a"
    assert devinfo._udev({"A": {"B": 1}}, "") == {"A": {"B": 1}}


def test_match():
    assert devinfo._match({"A": {"B": 1}}, {"a.b": 1})
    assert not devinfo._match({"A": {"B": 1}}, {"a.b": 2})
    assert devinfo._match({"A": {"B": 1}}, {"a.b": [1, 2]})
    assert not devinfo._match({"A": {"B": 1}}, {"a.b": [2, 3]})
    assert devinfo._match({"A": {"B": [1, 2]}}, {"a.b": 1})
    assert devinfo._match({"A": {"B": [1, 2]}}, {"a.b": [1, 3]})
    assert not devinfo._match({"A": {"B": [1, 2]}}, {"a.b": [3, 4]})
    assert devinfo._match({"A": 1}, {})


def test_devices():
    cdrom = {
        "S": ["dvd", "cdrom"],
        "E": {"ID_BUS": "ata"},
    }
    usb = {
        "E": {"ID_BUS": "usb"},
    }
    hd = {
        "E": {"ID_BUS": "ata"},
    }

    def udev_info(key):
        devices = {"sda": hd, "sdb": usb, "sr0": cdrom}
        return devices[key]

    with patch.dict(devinfo.__salt__, {"udev.info": udev_info}), patch.dict(
        devinfo.__grains__, {"disks": ["sda", "sdb", "sr0"]}
    ):
        assert devinfo.filter_({"e.id_bus": "ata"}, {}) == ["sda", "sr0"]
        assert devinfo.filter_({"e.id_bus": "usb"}, {}) == ["sdb"]
        assert devinfo.filter_({"e.id_bus": "ata"}, {"s": ["cdrom"]}) == ["sda"]


def test__hwinfo_parse_short():
    hwinfo = textwrap.dedent(
        """
        cpu:
                               QEMU Virtual CPU version 2.5+, 3591 MHz
        keyboard:
          /dev/input/event0    AT Translated Set 2 keyboard
        mouse:
          /dev/input/mice      VirtualPS/2 VMware VMMouse
          /dev/input/mice      VirtualPS/2 VMware VMMouse
        graphics card:
                               VGA compatible controller
        storage:
                               Floppy disk controller
                               Red Hat Qemu virtual machine
        network:
          ens3                 Virtio Ethernet Card 0
        network interface:
          lo                   Loopback network interface
          ens3                 Ethernet network interface
        disk:
          /dev/fd0             Disk
          /dev/sda             QEMU HARDDISK
        cdrom:
          /dev/sr0             QEMU DVD-ROM
        floppy:
          /dev/fd0             Floppy Disk
        bios:
                               BIOS
        bridge:
                               Red Hat Qemu virtual machine
                               Red Hat Qemu virtual machine
                               Red Hat Qemu virtual machine
        memory:
                               Main Memory
        unknown:
                               FPU
                               DMA controller
                               PIC
                               Keyboard controller
          /dev/lp0             Parallel controller
                               PS/2 Controller
                               Red Hat Virtio network device
          /dev/ttyS0           16550A
    """
    )
    assert devinfo._hwinfo_parse_short(hwinfo) == {
        "cpu": {0: "QEMU Virtual CPU version 2.5+, 3591 MHz"},
        "keyboard": {"/dev/input/event0": "AT Translated Set 2 keyboard"},
        "mouse": {"/dev/input/mice": "VirtualPS/2 VMware VMMouse"},
        "graphics card": {0: "VGA compatible controller"},
        "storage": {
            0: "Floppy disk controller",
            1: "Red Hat Qemu virtual machine",
        },
        "network": {"ens3": "Virtio Ethernet Card 0"},
        "network interface": {
            "lo": "Loopback network interface",
            "ens3": "Ethernet network interface",
        },
        "disk": {"/dev/fd0": "Disk", "/dev/sda": "QEMU HARDDISK"},
        "cdrom": {"/dev/sr0": "QEMU DVD-ROM"},
        "floppy": {"/dev/fd0": "Floppy Disk"},
        "bios": {0: "BIOS"},
        "bridge": {
            0: "Red Hat Qemu virtual machine",
            1: "Red Hat Qemu virtual machine",
            2: "Red Hat Qemu virtual machine",
        },
        "memory": {0: "Main Memory"},
        "unknown": {
            0: "FPU",
            1: "DMA controller",
            2: "PIC",
            3: "Keyboard controller",
            "/dev/lp0": "Parallel controller",
            4: "PS/2 Controller",
            5: "Red Hat Virtio network device",
            "/dev/ttyS0": "16550A",
        },
    }


def test__hwinfo_parse_full_floppy():
    hwinfo = textwrap.dedent(
        """
        01: None 00.0: 0102 Floppy disk controller
          [Created at floppy.112]
          Unique ID: rdCR.3wRL2_g4d2B
          Hardware Class: storage
          Model: "Floppy disk controller"
          I/O Port: 0x3f2 (rw)
          I/O Ports: 0x3f4-0x3f5 (rw)
          I/O Port: 0x3f7 (rw)
          DMA: 2
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        02: Floppy 00.0: 10603 Floppy Disk
          [Created at floppy.127]
          Unique ID: sPPV.oZ89vuho4Y3
          Parent ID: rdCR.3wRL2_g4d2B
          Hardware Class: floppy
          Model: "Floppy Disk"
          Device File: /dev/fd0
          Size: 3.5 ''
          Config Status: cfg=new, avail=yes, need=no, active=unknown
          Size: 5760 sectors a 512 bytes
          Capacity: 0 GB (2949120 bytes)
          Drive status: no medium
          Config Status: cfg=new, avail=yes, need=no, active=unknown
          Attached to: #1 (Floppy disk controller)
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "01": {
            "None 00.0": "0102 Floppy disk controller",
            "Note": "Created at floppy.112",
            "Unique ID": "rdCR.3wRL2_g4d2B",
            "Hardware Class": "storage",
            "Model": "Floppy disk controller",
            "I/O Ports": ["0x3f2 (rw)", "0x3f4-0x3f5 (rw)", "0x3f7 (rw)"],
            "DMA": "2",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "02": {
            "Floppy 00.0": "10603 Floppy Disk",
            "Note": "Created at floppy.127",
            "Unique ID": "sPPV.oZ89vuho4Y3",
            "Parent ID": "rdCR.3wRL2_g4d2B",
            "Hardware Class": "floppy",
            "Model": "Floppy Disk",
            "Device File": "/dev/fd0",
            "Size": ["3.5 ''", "5760 sectors a 512 bytes"],
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
            "Capacity": "0 GB (2949120 bytes)",
            "Drive status": "no medium",
            "Attached to": {"Handle": "#1 (Floppy disk controller)"},
        },
    }


def test__hwinfo_parse_full_bios():
    hwinfo = textwrap.dedent(
        """
        03: None 00.0: 10105 BIOS
          [Created at bios.186]
          Unique ID: rdCR.lZF+r4EgHp4
          Hardware Class: bios
          BIOS Keyboard LED Status:
            Scroll Lock: off
            Num Lock: off
            Caps Lock: off
          Serial Port 0: 0x3f8
          Parallel Port 0: 0x378
          Base Memory: 639 kB
          PnP BIOS: @@@0000
          MP spec rev 1.4 info:
            OEM id: "BOCHSCPU"
            Product id: "0.1"
            1 CPUs (0 disabled)
          BIOS32 Service Directory Entry: 0xfd2b0
          SMBIOS Version: 2.8
          BIOS Info: #0
            Vendor: "SeaBIOS"
            Version: "rel-1.12.1-0-ga5cab58e9a3f-prebuilt.qemu.org"
            Date: "04/01/2014"
            Start Address: 0xe8000
            ROM Size: 64 kB
            Features: 0x04000000000000000008
          System Info: #256
            Manufacturer: "QEMU"
            Product: "Standard PC (i440FX + PIIX, 1996)"
            Version: "pc-i440fx-4.0"
            UUID: undefined
            Wake-up: 0x06 (Power Switch)
          Chassis Info: #768
            Manufacturer: "QEMU"
            Version: "pc-i440fx-4.0"
            Type: 0x01 (Other)
            Bootup State: 0x03 (Safe)
            Power Supply State: 0x03 (Safe)
            Thermal State: 0x03 (Safe)
            Security Status: 0x02 (Unknown)
          Processor Info: #1024
            Socket: "CPU 0"
            Socket Type: 0x01 (Other)
            Socket Status: Populated
            Type: 0x03 (CPU)
            Family: 0x01 (Other)
            Manufacturer: "QEMU"
            Version: "pc-i440fx-4.0"
            Processor ID: 0x078bfbfd00000663
            Status: 0x01 (Enabled)
            Max. Speed: 2000 MHz
            Current Speed: 2000 MHz
          Physical Memory Array: #4096
            Use: 0x03 (System memory)
            Location: 0x01 (Other)
            Slots: 1
            Max. Size: 1 GB
            ECC: 0x06 (Multi-bit)
          Memory Device: #4352
            Location: "DIMM 0"
            Manufacturer: "QEMU"
            Memory Array: #4096
            Form Factor: 0x09 (DIMM)
            Type: 0x07 (RAM)
            Type Detail: 0x0002 (Other)
            Data Width: 0 bits
             Size: 1 GB
          Memory Array Mapping: #4864
            Memory Array: #4096
            Partition Width: 1
            Start Address: 0x00000000
            End Address: 0x40000000
          Type 32 Record: #8192
            Data 00: 20 0b 00 20 00 00 00 00 00 00 00
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "03": {
            "None 00.0": "10105 BIOS",
            "Note": "Created at bios.186",
            "Unique ID": "rdCR.lZF+r4EgHp4",
            "Hardware Class": "bios",
            "BIOS Keyboard LED Status": {
                "Scroll Lock": "off",
                "Num Lock": "off",
                "Caps Lock": "off",
            },
            "Serial Port 0": "0x3f8",
            "Parallel Port 0": "0x378",
            "Base Memory": "639 kB",
            "PnP BIOS": "@@@0000",
            "MP spec rev 1.4 info": {
                "OEM id": "BOCHSCPU",
                "Product id": "0.1",
                "Note": "1 CPUs (0 disabled)",
            },
            "BIOS32 Service Directory Entry": "0xfd2b0",
            "SMBIOS Version": "2.8",
            "BIOS Info": {
                "Handle": "#0",
                "Vendor": "SeaBIOS",
                "Version": "rel-1.12.1-0-ga5cab58e9a3f-prebuilt.qemu.org",
                "Date": "04/01/2014",
                "Start Address": "0xe8000",
                "ROM Size": "64 kB",
                "Features": ["0x04000000000000000008"],
            },
            "System Info": {
                "Handle": "#256",
                "Manufacturer": "QEMU",
                "Product": "Standard PC (i440FX + PIIX, 1996)",
                "Version": "pc-i440fx-4.0",
                "UUID": "undefined",
                "Wake-up": "0x06 (Power Switch)",
            },
            "Chassis Info": {
                "Handle": "#768",
                "Manufacturer": "QEMU",
                "Version": "pc-i440fx-4.0",
                "Type": "0x01 (Other)",
                "Bootup State": "0x03 (Safe)",
                "Power Supply State": "0x03 (Safe)",
                "Thermal State": "0x03 (Safe)",
                "Security Status": "0x02 (Unknown)",
            },
            "Processor Info": {
                "Handle": "#1024",
                "Socket": "CPU 0",
                "Socket Type": "0x01 (Other)",
                "Socket Status": "Populated",
                "Type": "0x03 (CPU)",
                "Family": "0x01 (Other)",
                "Manufacturer": "QEMU",
                "Version": "pc-i440fx-4.0",
                "Processor ID": "0x078bfbfd00000663",
                "Status": "0x01 (Enabled)",
                "Max. Speed": "2000 MHz",
                "Current Speed": "2000 MHz",
            },
            "Physical Memory Array": {
                "Handle": "#4096",
                "Use": "0x03 (System memory)",
                "Location": "0x01 (Other)",
                "Slots": "1",
                "Max. Size": "1 GB",
                "ECC": "0x06 (Multi-bit)",
            },
            "Memory Device": {
                "Handle": "#4352",
                "Location": "DIMM 0",
                "Manufacturer": "QEMU",
                "Memory Array": {"Handle": "#4096"},
                "Form Factor": "0x09 (DIMM)",
                "Type": "0x07 (RAM)",
                "Type Detail": "0x0002 (Other)",
                "Data Width": "0 bits",
                "Size": "1 GB",
            },
            "Memory Array Mapping": {
                "Handle": "#4864",
                "Memory Array": {"Handle": "#4096"},
                "Partition Width": "1",
                "Start Address": "0x00000000",
                "End Address": "0x40000000",
            },
            "Type 32 Record": {
                "Handle": "#8192",
                "Data 00": "20 0b 00 20 00 00 00 00 00 00 00",
            },
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_system():
    hwinfo = textwrap.dedent(
        """
        04: None 00.0: 10107 System
          [Created at sys.64]
          Unique ID: rdCR.n_7QNeEnh23
          Hardware Class: system
          Model: "System"
          Formfactor: "desktop"
          Driver Info #0:
            Driver Status: thermal,fan are not active
            Driver Activation Cmd: "modprobe thermal; modprobe fan"
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "04": {
            "None 00.0": "10107 System",
            "Note": "Created at sys.64",
            "Unique ID": "rdCR.n_7QNeEnh23",
            "Hardware Class": "system",
            "Model": "System",
            "Formfactor": "desktop",
            "Driver Info #0": {
                "Driver Status": "thermal,fan are not active",
                "Driver Activation Cmd": "modprobe thermal; modprobe fan",
            },
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_unknown():
    hwinfo = textwrap.dedent(
        """
        05: None 00.0: 10104 FPU
          [Created at misc.191]
          Unique ID: rdCR.EMpH5pjcahD
          Hardware Class: unknown
          Model: "FPU"
          I/O Ports: 0xf0-0xff (rw)
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        06: None 00.0: 0801 DMA controller (8237)
          [Created at misc.205]
          Unique ID: rdCR.f5u1ucRm+H9
          Hardware Class: unknown
          Model: "DMA controller"
          I/O Ports: 0x00-0xcf7 (rw)
          I/O Ports: 0xc0-0xdf (rw)
          I/O Ports: 0x80-0x8f (rw)
          DMA: 4
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        07: None 00.0: 0800 PIC (8259)
          [Created at misc.218]
          Unique ID: rdCR.8uRK7LxiIA2
          Hardware Class: unknown
          Model: "PIC"
          I/O Ports: 0x20-0x21 (rw)
          I/O Ports: 0xa0-0xa1 (rw)
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        08: None 00.0: 0900 Keyboard controller
          [Created at misc.250]
          Unique ID: rdCR.9N+EecqykME
          Hardware Class: unknown
          Model: "Keyboard controller"
          I/O Port: 0x60 (rw)
          I/O Port: 0x64 (rw)
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        09: None 00.0: 0701 Parallel controller (SPP)
          [Created at misc.261]
          Unique ID: YMnp.ecK7NLYWZ5D
          Hardware Class: unknown
          Model: "Parallel controller"
          Device File: /dev/lp0
          I/O Ports: 0x378-0x37a (rw)
          I/O Ports: 0x37b-0x37f (rw)
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        10: None 00.0: 10400 PS/2 Controller
          [Created at misc.303]
          Unique ID: rdCR.DziBbWO85o5
          Hardware Class: unknown
          Model: "PS/2 Controller"
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "05": {
            "None 00.0": "10104 FPU",
            "Note": "Created at misc.191",
            "Unique ID": "rdCR.EMpH5pjcahD",
            "Hardware Class": "unknown",
            "Model": "FPU",
            "I/O Ports": "0xf0-0xff (rw)",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "06": {
            "None 00.0": "0801 DMA controller (8237)",
            "Note": "Created at misc.205",
            "Unique ID": "rdCR.f5u1ucRm+H9",
            "Hardware Class": "unknown",
            "Model": "DMA controller",
            "I/O Ports": [
                "0x00-0xcf7 (rw)",
                "0xc0-0xdf (rw)",
                "0x80-0x8f (rw)",
            ],
            "DMA": "4",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "07": {
            "None 00.0": "0800 PIC (8259)",
            "Note": "Created at misc.218",
            "Unique ID": "rdCR.8uRK7LxiIA2",
            "Hardware Class": "unknown",
            "Model": "PIC",
            "I/O Ports": ["0x20-0x21 (rw)", "0xa0-0xa1 (rw)"],
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "08": {
            "None 00.0": "0900 Keyboard controller",
            "Note": "Created at misc.250",
            "Unique ID": "rdCR.9N+EecqykME",
            "Hardware Class": "unknown",
            "Model": "Keyboard controller",
            "I/O Ports": ["0x60 (rw)", "0x64 (rw)"],
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "09": {
            "None 00.0": "0701 Parallel controller (SPP)",
            "Note": "Created at misc.261",
            "Unique ID": "YMnp.ecK7NLYWZ5D",
            "Hardware Class": "unknown",
            "Model": "Parallel controller",
            "Device File": "/dev/lp0",
            "I/O Ports": ["0x378-0x37a (rw)", "0x37b-0x37f (rw)"],
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "10": {
            "None 00.0": "10400 PS/2 Controller",
            "Note": "Created at misc.303",
            "Unique ID": "rdCR.DziBbWO85o5",
            "Hardware Class": "unknown",
            "Model": "PS/2 Controller",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_memory():
    hwinfo = textwrap.dedent(
        """
        12: None 00.0: 10102 Main Memory
          [Created at memory.74]
          Unique ID: rdCR.CxwsZFjVASF
          Hardware Class: memory
          Model: "Main Memory"
          Memory Range: 0x00000000-0x3cefffff (rw)
          Memory Size: 960 MB
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "12": {
            "None 00.0": "10102 Main Memory",
            "Note": "Created at memory.74",
            "Unique ID": "rdCR.CxwsZFjVASF",
            "Hardware Class": "memory",
            "Model": "Main Memory",
            "Memory Range": "0x00000000-0x3cefffff (rw)",
            "Memory Size": "960 MB",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_bridge():
    hwinfo = textwrap.dedent(
        """
        13: PCI 01.0: 0601 ISA bridge
          [Created at pci.386]
          Unique ID: vSkL.ucdhKwLeeAA
          SysFS ID: /devices/pci0000:00/0000:00:01.0
          SysFS BusID: 0000:00:01.0
          Hardware Class: bridge
          Model: "Red Hat Qemu virtual machine"
          Vendor: pci 0x8086 "Intel Corporation"
          Device: pci 0x7000 "82371SB PIIX3 ISA [Natoma/Triton II]"
          SubVendor: pci 0x1af4 "Red Hat, Inc."
          SubDevice: pci 0x1100 "Qemu virtual machine"
          Module Alias: "pci:v00008086d00007000sv00001AF4sd00001100bc06sc01i00"
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        14: PCI 00.0: 0600 Host bridge
          [Created at pci.386]
          Unique ID: qLht.YeL3TKDjrxE
          SysFS ID: /devices/pci0000:00/0000:00:00.0
          SysFS BusID: 0000:00:00.0
          Hardware Class: bridge
          Model: "Red Hat Qemu virtual machine"
          Vendor: pci 0x8086 "Intel Corporation"
          Device: pci 0x1237 "440FX - 82441FX PMC [Natoma]"
          SubVendor: pci 0x1af4 "Red Hat, Inc."
          SubDevice: pci 0x1100 "Qemu virtual machine"
          Revision: 0x02
          Module Alias: "pci:v00008086d00001237sv00001AF4sd00001100bc06sc00i00"
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        15: PCI 01.3: 0680 Bridge
          [Created at pci.386]
          Unique ID: VRCs.M9Cc8lcQjE2
          SysFS ID: /devices/pci0000:00/0000:00:01.3
          SysFS BusID: 0000:00:01.3
          Hardware Class: bridge
          Model: "Red Hat Qemu virtual machine"
          Vendor: pci 0x8086 "Intel Corporation"
          Device: pci 0x7113 "82371AB/EB/MB PIIX4 ACPI"
          SubVendor: pci 0x1af4 "Red Hat, Inc."
          SubDevice: pci 0x1100 "Qemu virtual machine"
          Revision: 0x03
          Driver: "piix4_smbus"
          Driver Modules: "i2c_piix4"
          IRQ: 9 (no events)
          Module Alias: "pci:v00008086d00007113sv00001AF4sd00001100bc06sc80i00"
          Driver Info #0:
            Driver Status: i2c_piix4 is active
            Driver Activation Cmd: "modprobe i2c_piix4"
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "13": {
            "PCI 01.0": "0601 ISA bridge",
            "Note": "Created at pci.386",
            "Unique ID": "vSkL.ucdhKwLeeAA",
            "SysFS ID": "/devices/pci0000:00/0000:00:01.0",
            "SysFS BusID": "0000:00:01.0",
            "Hardware Class": "bridge",
            "Model": "Red Hat Qemu virtual machine",
            "Vendor": 'pci 0x8086 "Intel Corporation"',
            "Device": 'pci 0x7000 "82371SB PIIX3 ISA [Natoma/Triton II]"',
            "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
            "SubDevice": 'pci 0x1100 "Qemu virtual machine"',
            "Module Alias": ("pci:v00008086d00007000sv00001AF4sd00001100bc06sc01i00"),
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "14": {
            "PCI 00.0": "0600 Host bridge",
            "Note": "Created at pci.386",
            "Unique ID": "qLht.YeL3TKDjrxE",
            "SysFS ID": "/devices/pci0000:00/0000:00:00.0",
            "SysFS BusID": "0000:00:00.0",
            "Hardware Class": "bridge",
            "Model": "Red Hat Qemu virtual machine",
            "Vendor": 'pci 0x8086 "Intel Corporation"',
            "Device": 'pci 0x1237 "440FX - 82441FX PMC [Natoma]"',
            "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
            "SubDevice": 'pci 0x1100 "Qemu virtual machine"',
            "Revision": "0x02",
            "Module Alias": ("pci:v00008086d00001237sv00001AF4sd00001100bc06sc00i00"),
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "15": {
            "PCI 01.3": "0680 Bridge",
            "Note": "Created at pci.386",
            "Unique ID": "VRCs.M9Cc8lcQjE2",
            "SysFS ID": "/devices/pci0000:00/0000:00:01.3",
            "SysFS BusID": "0000:00:01.3",
            "Hardware Class": "bridge",
            "Model": "Red Hat Qemu virtual machine",
            "Vendor": 'pci 0x8086 "Intel Corporation"',
            "Device": 'pci 0x7113 "82371AB/EB/MB PIIX4 ACPI"',
            "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
            "SubDevice": 'pci 0x1100 "Qemu virtual machine"',
            "Revision": "0x03",
            "Driver": ["piix4_smbus"],
            "Driver Modules": ["i2c_piix4"],
            "IRQ": "9 (no events)",
            "Module Alias": ("pci:v00008086d00007113sv00001AF4sd00001100bc06sc80i00"),
            "Driver Info #0": {
                "Driver Status": "i2c_piix4 is active",
                "Driver Activation Cmd": "modprobe i2c_piix4",
            },
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_ethernet():
    hwinfo = textwrap.dedent(
        """
        16: PCI 03.0: 0200 Ethernet controller
          [Created at pci.386]
          Unique ID: 3hqH.pkM7KXDR457
          SysFS ID: /devices/pci0000:00/0000:00:03.0
          SysFS BusID: 0000:00:03.0
          Hardware Class: unknown
          Model: "Red Hat Virtio network device"
          Vendor: pci 0x1af4 "Red Hat, Inc."
          Device: pci 0x1000 "Virtio network device"
          SubVendor: pci 0x1af4 "Red Hat, Inc."
          SubDevice: pci 0x0001
          Driver: "virtio-pci"
          Driver Modules: "virtio_pci"
          I/O Ports: 0xc000-0xc01f (rw)
          Memory Range: 0xfebd1000-0xfebd1fff (rw,non-prefetchable)
          Memory Range: 0xfe000000-0xfe003fff (ro,non-prefetchable)
          Memory Range: 0xfeb80000-0xfebbffff (ro,non-prefetchable,disabled)
          IRQ: 11 (no events)
          Module Alias: "pci:v00001AF4d00001000sv00001AF4sd00000001bc02sc00i00"
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "16": {
            "PCI 03.0": "0200 Ethernet controller",
            "Note": "Created at pci.386",
            "Unique ID": "3hqH.pkM7KXDR457",
            "SysFS ID": "/devices/pci0000:00/0000:00:03.0",
            "SysFS BusID": "0000:00:03.0",
            "Hardware Class": "unknown",
            "Model": "Red Hat Virtio network device",
            "Vendor": 'pci 0x1af4 "Red Hat, Inc."',
            "Device": 'pci 0x1000 "Virtio network device"',
            "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
            "SubDevice": "pci 0x0001",
            "Driver": ["virtio-pci"],
            "Driver Modules": ["virtio_pci"],
            "I/O Ports": "0xc000-0xc01f (rw)",
            "Memory Range": [
                "0xfebd1000-0xfebd1fff (rw,non-prefetchable)",
                "0xfe000000-0xfe003fff (ro,non-prefetchable)",
                "0xfeb80000-0xfebbffff (ro,non-prefetchable,disabled)",
            ],
            "IRQ": "11 (no events)",
            "Module Alias": ("pci:v00001AF4d00001000sv00001AF4sd00000001bc02sc00i00"),
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_storage():
    hwinfo = textwrap.dedent(
        """
        17: PCI 01.1: 0101 IDE interface (ISA Compatibility mode-only controller, supports bus mts bus mastering)
          [Created at pci.386]
          Unique ID: mnDB.3sKqaxiizg6
          SysFS ID: /devices/pci0000:00/0000:00:01.1
          SysFS BusID: 0000:00:01.1
          Hardware Class: storage
          Model: "Red Hat Qemu virtual machine"
          Vendor: pci 0x8086 "Intel Corporation"
          Device: pci 0x7010 "82371SB PIIX3 IDE [Natoma/Triton II]"
          SubVendor: pci 0x1af4 "Red Hat, Inc."
          SubDevice: pci 0x1100 "Qemu virtual machine"
          Driver: "ata_piix"
          Driver Modules: "ata_piix"
          I/O Ports: 0x1f0-0x1f7 (rw)
          I/O Port: 0x3f6 (rw)
          I/O Ports: 0x170-0x177 (rw)
          I/O Port: 0x376 (rw)
          I/O Ports: 0xc020-0xc02f (rw)
          Module Alias: "pci:v00008086d00007010sv00001AF4sd00001100bc01sc01i80"
          Driver Info #0:
            Driver Status: ata_piix is active
            Driver Activation Cmd: "modprobe ata_piix"
          Driver Info #1:
            Driver Status: ata_generic is active
            Driver Activation Cmd: "modprobe ata_generic"
          Driver Info #2:
            Driver Status: pata_acpi is active
            Driver Activation Cmd: "modprobe pata_acpi"
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "17": {
            "PCI 01.1": (
                "0101 IDE interface (ISA Compatibility mode-only controller,"
                " supports bus mts bus mastering)"
            ),
            "Note": "Created at pci.386",
            "Unique ID": "mnDB.3sKqaxiizg6",
            "SysFS ID": "/devices/pci0000:00/0000:00:01.1",
            "SysFS BusID": "0000:00:01.1",
            "Hardware Class": "storage",
            "Model": "Red Hat Qemu virtual machine",
            "Vendor": 'pci 0x8086 "Intel Corporation"',
            "Device": 'pci 0x7010 "82371SB PIIX3 IDE [Natoma/Triton II]"',
            "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
            "SubDevice": 'pci 0x1100 "Qemu virtual machine"',
            "Driver": ["ata_piix"],
            "Driver Modules": ["ata_piix"],
            "I/O Ports": [
                "0x1f0-0x1f7 (rw)",
                "0x3f6 (rw)",
                "0x170-0x177 (rw)",
                "0x376 (rw)",
                "0xc020-0xc02f (rw)",
            ],
            "Module Alias": ("pci:v00008086d00007010sv00001AF4sd00001100bc01sc01i80"),
            "Driver Info #0": {
                "Driver Status": "ata_piix is active",
                "Driver Activation Cmd": "modprobe ata_piix",
            },
            "Driver Info #1": {
                "Driver Status": "ata_generic is active",
                "Driver Activation Cmd": "modprobe ata_generic",
            },
            "Driver Info #2": {
                "Driver Status": "pata_acpi is active",
                "Driver Activation Cmd": "modprobe pata_acpi",
            },
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_video():
    hwinfo = textwrap.dedent(
        """
        18: PCI 02.0: 0300 VGA compatible controller (VGA)
          [Created at pci.386]
          Unique ID: _Znp.WspiKb87LiA
          SysFS ID: /devices/pci0000:00/0000:00:02.0
          SysFS BusID: 0000:00:02.0
          Hardware Class: graphics card
          Model: "VGA compatible controller"
          Vendor: pci 0x1234
          Device: pci 0x1111
          SubVendor: pci 0x1af4 "Red Hat, Inc."
          SubDevice: pci 0x1100
          Revision: 0x02
          Driver: "bochs-drm"
          Driver Modules: "bochs_drm"
          Memory Range: 0xfd000000-0xfdffffff (ro,non-prefetchable)
          Memory Range: 0xfebd0000-0xfebd0fff (rw,non-prefetchable)
          Memory Range: 0x000c0000-0x000dffff (rw,non-prefetchable,disabled)
          Module Alias: "pci:v00001234d00001111sv00001AF4sd00001100bc03sc00i00"
          Driver Info #0:
            Driver Status: bochs_drm is active
            Driver Activation Cmd: "modprobe bochs_drm"
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "18": {
            "PCI 02.0": "0300 VGA compatible controller (VGA)",
            "Note": "Created at pci.386",
            "Unique ID": "_Znp.WspiKb87LiA",
            "SysFS ID": "/devices/pci0000:00/0000:00:02.0",
            "SysFS BusID": "0000:00:02.0",
            "Hardware Class": "graphics card",
            "Model": "VGA compatible controller",
            "Vendor": "pci 0x1234",
            "Device": "pci 0x1111",
            "SubVendor": 'pci 0x1af4 "Red Hat, Inc."',
            "SubDevice": "pci 0x1100",
            "Revision": "0x02",
            "Driver": ["bochs-drm"],
            "Driver Modules": ["bochs_drm"],
            "Memory Range": [
                "0xfd000000-0xfdffffff (ro,non-prefetchable)",
                "0xfebd0000-0xfebd0fff (rw,non-prefetchable)",
                "0x000c0000-0x000dffff (rw,non-prefetchable,disabled)",
            ],
            "Module Alias": ("pci:v00001234d00001111sv00001AF4sd00001100bc03sc00i00"),
            "Driver Info #0": {
                "Driver Status": "bochs_drm is active",
                "Driver Activation Cmd": "modprobe bochs_drm",
            },
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_network():
    hwinfo = textwrap.dedent(
        """
        19: Virtio 00.0: 0200 Ethernet controller
          [Created at pci.1679]
          Unique ID: vWuh.VIRhsc57kTD
          Parent ID: 3hqH.pkM7KXDR457
          SysFS ID: /devices/pci0000:00/0000:00:03.0/virtio0
          SysFS BusID: virtio0
          Hardware Class: network
          Model: "Virtio Ethernet Card 0"
          Vendor: int 0x6014 "Virtio"
          Device: int 0x0001 "Ethernet Card 0"
          Driver: "virtio_net"
          Driver Modules: "virtio_net"
          Device File: ens3
          HW Address: 52:54:00:12:34:56
          Permanent HW Address: 52:54:00:12:34:56
          Link detected: yes
          Module Alias: "virtio:d00000001v00001AF4"
          Driver Info #0:
            Driver Status: virtio_net is active
            Driver Activation Cmd: "modprobe virtio_net"
          Config Status: cfg=new, avail=yes, need=no, active=unknown
          Attached to: #16 (Ethernet controller)

        20: None 00.0: 0700 Serial controller (16550)
          [Created at serial.74]
          Unique ID: S_Uw.3fyvFV+mbWD
          Hardware Class: unknown
          Model: "16550A"
          Device: "16550A"
          Device File: /dev/ttyS0
          Tags: mouse, modem, braille
          I/O Ports: 0x3f8-0x3ff (rw)
          IRQ: 4 (55234 events)
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "19": {
            "Virtio 00.0": "0200 Ethernet controller",
            "Note": "Created at pci.1679",
            "Unique ID": "vWuh.VIRhsc57kTD",
            "Parent ID": "3hqH.pkM7KXDR457",
            "SysFS ID": "/devices/pci0000:00/0000:00:03.0/virtio0",
            "SysFS BusID": "virtio0",
            "Hardware Class": "network",
            "Model": "Virtio Ethernet Card 0",
            "Vendor": 'int 0x6014 "Virtio"',
            "Device": 'int 0x0001 "Ethernet Card 0"',
            "Driver": ["virtio_net"],
            "Driver Modules": ["virtio_net"],
            "Device File": "ens3",
            "HW Address": "52:54:00:12:34:56",
            "Permanent HW Address": "52:54:00:12:34:56",
            "Link detected": "yes",
            "Module Alias": "virtio:d00000001v00001AF4",
            "Driver Info #0": {
                "Driver Status": "virtio_net is active",
                "Driver Activation Cmd": "modprobe virtio_net",
            },
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
            "Attached to": {"Handle": "#16 (Ethernet controller)"},
        },
        "20": {
            "None 00.0": "0700 Serial controller (16550)",
            "Note": "Created at serial.74",
            "Unique ID": "S_Uw.3fyvFV+mbWD",
            "Hardware Class": "unknown",
            "Model": "16550A",
            "Device": "16550A",
            "Device File": "/dev/ttyS0",
            "Tags": ["mouse", "modem", "braille"],
            "I/O Ports": "0x3f8-0x3ff (rw)",
            "IRQ": "4 (55234 events)",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_disk():
    hwinfo = textwrap.dedent(
        """
        21: SCSI 100.0: 10602 CD-ROM (DVD)
          [Created at block.249]
          Unique ID: KD9E.53N0UD4ozwD
          Parent ID: mnDB.3sKqaxiizg6
          SysFS ID: /class/block/sr0
          SysFS BusID: 1:0:0:0
          SysFS Device Link: /devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0/1:0:0:0
          Hardware Class: cdrom
          Model: "QEMU DVD-ROM"
          Vendor: "QEMU"
          Device: "QEMU DVD-ROM"
          Revision: "2.5+"
          Driver: "ata_piix", "sr"
          Driver Modules: "ata_piix", "sr_mod"
          Device File: /dev/sr0 (/dev/sg1)
          Device Files: /dev/sr0, /dev/cdrom, /dev/dvd, /dev/disk/by-path/pci-0000:00:01.1-ata-2, /dev/disk/by-id/ata-QEMU_DVD-ROM_QM00003, /dev/disk/by-uuid/2019-08-11-11-44-39-00, /dev/disk/by-label/CDROM
          Device Number: block 11:0 (char 21:1)
          Features: DVD, MRW, MRW-W
          Config Status: cfg=new, avail=yes, need=no, active=unknown
          Attached to: #17 (IDE interface)
          Drive Speed: 4
          Volume ID: "CDROM"
          Application: "0X5228779D"
          Publisher: "SUSE LINUX GMBH"
          Preparer: "KIWI - HTTPS://GITHUB.COM/OSINSIDE/KIWI"
          Creation date: "2019081111443900"
          El Torito info: platform 0, bootable
            Boot Catalog: at sector 0x00fa
            Media: none starting at sector 0x00fb
            Load: 2048 bytes

        22: None 00.0: 10600 Disk
          [Created at block.245]
          Unique ID: kwWm.Fxp0d3BezAE
          SysFS ID: /class/block/fd0
          SysFS BusID: floppy.0
          SysFS Device Link: /devices/platform/floppy.0
          Hardware Class: disk
          Model: "Disk"
          Driver: "floppy"
          Driver Modules: "floppy"
          Device File: /dev/fd0
          Device Number: block 2:0
          Size: 8 sectors a 512 bytes
          Capacity: 0 GB (4096 bytes)
          Drive status: no medium
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        23: IDE 00.0: 10600 Disk
          [Created at block.245]
          Unique ID: 3OOL.W8iGvCekDp8
          Parent ID: mnDB.3sKqaxiizg6
          SysFS ID: /class/block/sda
          SysFS BusID: 0:0:0:0
          SysFS Device Link: /devices/pci0000:00/0000:00:01.1/ata1/host0/target0:0:0/0:0:0:0
          Hardware Class: disk
          Model: "QEMU HARDDISK"
          Vendor: "QEMU"
          Device: "HARDDISK"
          Revision: "2.5+"
          Serial ID: "QM00001"
          Driver: "ata_piix", "sd"
          Driver Modules: "ata_piix"
          Device File: /dev/sda
          Device Files: /dev/sda, /dev/disk/by-path/pci-0000:00:01.1-ata-1, /dev/disk/by-id/ata-QEMU_HARDDISK_QM00001
          Device Number: block 8:0-8:15
          Geometry (Logical): CHS 3133/255/63
          Size: 50331648 sectors a 512 bytes
          Capacity: 24 GB (25769803776 bytes)
          Config Status: cfg=new, avail=yes, need=no, active=unknown
          Attached to: #17 (IDE interface)
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "21": {
            "SCSI 100.0": "10602 CD-ROM (DVD)",
            "Note": "Created at block.249",
            "Unique ID": "KD9E.53N0UD4ozwD",
            "Parent ID": "mnDB.3sKqaxiizg6",
            "SysFS ID": "/class/block/sr0",
            "SysFS BusID": "1:0:0:0",
            "SysFS Device Link": "/devices/pci0000:00/0000:00:01.1/ata2/host1/target1:0:0/1:0:0:0",
            "Hardware Class": "cdrom",
            "Model": "QEMU DVD-ROM",
            "Vendor": "QEMU",
            "Device": "QEMU DVD-ROM",
            "Revision": "2.5+",
            "Driver": ["ata_piix", "sr"],
            "Driver Modules": ["ata_piix", "sr_mod"],
            "Device File": "/dev/sr0 (/dev/sg1)",
            "Device Files": [
                "/dev/sr0",
                "/dev/cdrom",
                "/dev/dvd",
                "/dev/disk/by-path/pci-0000:00:01.1-ata-2",
                "/dev/disk/by-id/ata-QEMU_DVD-ROM_QM00003",
                "/dev/disk/by-uuid/2019-08-11-11-44-39-00",
                "/dev/disk/by-label/CDROM",
            ],
            "Device Number": "block 11:0 (char 21:1)",
            "Features": ["DVD", "MRW", "MRW-W"],
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
            "Attached to": {"Handle": "#17 (IDE interface)"},
            "Drive Speed": "4",
            "Volume ID": "CDROM",
            "Application": "0X5228779D",
            "Publisher": "SUSE LINUX GMBH",
            "Preparer": "KIWI - HTTPS://GITHUB.COM/OSINSIDE/KIWI",
            "Creation date": "2019081111443900",
            "El Torito info": {
                "platform": "0",
                "bootable": "yes",
                "Boot Catalog": "at sector 0x00fa",
                "Media": "none starting at sector 0x00fb",
                "Load": "2048 bytes",
            },
        },
        "22": {
            "None 00.0": "10600 Disk",
            "Note": "Created at block.245",
            "Unique ID": "kwWm.Fxp0d3BezAE",
            "SysFS ID": "/class/block/fd0",
            "SysFS BusID": "floppy.0",
            "SysFS Device Link": "/devices/platform/floppy.0",
            "Hardware Class": "disk",
            "Model": "Disk",
            "Driver": ["floppy"],
            "Driver Modules": ["floppy"],
            "Device File": "/dev/fd0",
            "Device Number": "block 2:0",
            "Size": "8 sectors a 512 bytes",
            "Capacity": "0 GB (4096 bytes)",
            "Drive status": "no medium",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "23": {
            "IDE 00.0": "10600 Disk",
            "Note": "Created at block.245",
            "Unique ID": "3OOL.W8iGvCekDp8",
            "Parent ID": "mnDB.3sKqaxiizg6",
            "SysFS ID": "/class/block/sda",
            "SysFS BusID": "0:0:0:0",
            "SysFS Device Link": "/devices/pci0000:00/0000:00:01.1/ata1/host0/target0:0:0/0:0:0:0",
            "Hardware Class": "disk",
            "Model": "QEMU HARDDISK",
            "Vendor": "QEMU",
            "Device": "HARDDISK",
            "Revision": "2.5+",
            "Serial ID": "QM00001",
            "Driver": ["ata_piix", "sd"],
            "Driver Modules": ["ata_piix"],
            "Device File": "/dev/sda",
            "Device Files": [
                "/dev/sda",
                "/dev/disk/by-path/pci-0000:00:01.1-ata-1",
                "/dev/disk/by-id/ata-QEMU_HARDDISK_QM00001",
            ],
            "Device Number": "block 8:0-8:15",
            "Geometry (Logical)": "CHS 3133/255/63",
            "Size": "50331648 sectors a 512 bytes",
            "Capacity": "24 GB (25769803776 bytes)",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
            "Attached to": {"Handle": "#17 (IDE interface)"},
        },
    }


def test__hwinfo_parse_full_keyboard():
    hwinfo = textwrap.dedent(
        """
        24: PS/2 00.0: 10800 Keyboard
          [Created at input.226]
          Unique ID: nLyy.+49ps10DtUF
          Hardware Class: keyboard
          Model: "AT Translated Set 2 keyboard"
          Vendor: 0x0001
          Device: 0x0001 "AT Translated Set 2 keyboard"
          Compatible to: int 0x0211 0x0001
          Device File: /dev/input/event0
          Device Files: /dev/input/event0, /dev/input/by-path/platform-i8042-serio-0-event-kbd
          Device Number: char 13:64
          Driver Info #0:
            XkbRules: xfree86
            XkbModel: pc104
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "24": {
            "PS/2 00.0": "10800 Keyboard",
            "Note": "Created at input.226",
            "Unique ID": "nLyy.+49ps10DtUF",
            "Hardware Class": "keyboard",
            "Model": "AT Translated Set 2 keyboard",
            "Vendor": "0x0001",
            "Device": '0x0001 "AT Translated Set 2 keyboard"',
            "Compatible to": "int 0x0211 0x0001",
            "Device File": "/dev/input/event0",
            "Device Files": [
                "/dev/input/event0",
                "/dev/input/by-path/platform-i8042-serio-0-event-kbd",
            ],
            "Device Number": "char 13:64",
            "Driver Info #0": {"XkbRules": "xfree86", "XkbModel": "pc104"},
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_mouse():
    hwinfo = textwrap.dedent(
        """
        25: PS/2 00.0: 10500 PS/2 Mouse
          [Created at input.249]
          Unique ID: AH6Q.mYF0pYoTCW7
          Hardware Class: mouse
          Model: "VirtualPS/2 VMware VMMouse"
          Vendor: 0x0002
          Device: 0x0013 "VirtualPS/2 VMware VMMouse"
          Compatible to: int 0x0210 0x0003
          Device File: /dev/input/mice (/dev/input/mouse0)
          Device Files: /dev/input/mice, /dev/input/mouse0, /dev/input/event1, /dev/input/by-path/platform-i8042-serio-1-event-mouse, /dev/input/by-path/platform-i8042-serio-1-mouse
          Device Number: char 13:63 (char 13:32)
          Driver Info #0:
            Buttons: 3
            Wheels: 0
            XFree86 Protocol: explorerps/2
            GPM Protocol: exps2
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        26: PS/2 00.0: 10500 PS/2 Mouse
          [Created at input.249]
          Unique ID: AH6Q.++hSeDccb2F
          Hardware Class: mouse
          Model: "VirtualPS/2 VMware VMMouse"
          Vendor: 0x0002
          Device: 0x0013 "VirtualPS/2 VMware VMMouse"
          Compatible to: int 0x0210 0x0012
          Device File: /dev/input/mice (/dev/input/mouse1)
          Device Files: /dev/input/mice, /dev/input/mouse1, /dev/input/event2
          Device Number: char 13:63 (char 13:33)
          Driver Info #0:
            Buttons: 2
            Wheels: 1
            XFree86 Protocol: explorerps/2
            GPM Protocol: exps2
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "25": {
            "PS/2 00.0": "10500 PS/2 Mouse",
            "Note": "Created at input.249",
            "Unique ID": "AH6Q.mYF0pYoTCW7",
            "Hardware Class": "mouse",
            "Model": "VirtualPS/2 VMware VMMouse",
            "Vendor": "0x0002",
            "Device": '0x0013 "VirtualPS/2 VMware VMMouse"',
            "Compatible to": "int 0x0210 0x0003",
            "Device File": "/dev/input/mice (/dev/input/mouse0)",
            "Device Files": [
                "/dev/input/mice",
                "/dev/input/mouse0",
                "/dev/input/event1",
                "/dev/input/by-path/platform-i8042-serio-1-event-mouse",
                "/dev/input/by-path/platform-i8042-serio-1-mouse",
            ],
            "Device Number": "char 13:63 (char 13:32)",
            "Driver Info #0": {
                "Buttons": "3",
                "Wheels": "0",
                "XFree86 Protocol": "explorerps/2",
                "GPM Protocol": "exps2",
            },
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "26": {
            "PS/2 00.0": "10500 PS/2 Mouse",
            "Note": "Created at input.249",
            "Unique ID": "AH6Q.++hSeDccb2F",
            "Hardware Class": "mouse",
            "Model": "VirtualPS/2 VMware VMMouse",
            "Vendor": "0x0002",
            "Device": '0x0013 "VirtualPS/2 VMware VMMouse"',
            "Compatible to": "int 0x0210 0x0012",
            "Device File": "/dev/input/mice (/dev/input/mouse1)",
            "Device Files": [
                "/dev/input/mice",
                "/dev/input/mouse1",
                "/dev/input/event2",
            ],
            "Device Number": "char 13:63 (char 13:33)",
            "Driver Info #0": {
                "Buttons": "2",
                "Wheels": "1",
                "XFree86 Protocol": "explorerps/2",
                "GPM Protocol": "exps2",
            },
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_cpu():
    hwinfo = textwrap.dedent(
        """
        27: None 00.0: 10103 CPU
          [Created at cpu.462]
          Unique ID: rdCR.j8NaKXDZtZ6
          Hardware Class: cpu
          Arch: X86-64
          Vendor: "GenuineIntel"
          Model: 6.6.3 "QEMU Virtual CPU version 2.5+"
          Features: fpu,de,pse,tsc,msr,pae,mce,cx8,apic,sep,mtrr,pge,mca,cmov,pse36,clflush,mmx,fxsr,sse,sse2,syscall,nx,lm,rep_good,nopl,xtopology,cpuid,tsc_known_freq,pni,cx16,x2apic,hypervisor,lahf_lm,cpuid_fault,pti
          Clock: 3591 MHz
          BogoMips: 7182.68
          Cache: 16384 kb
          Config Status: cfg=new, avail=yes, need=no, active=unknown
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "27": {
            "None 00.0": "10103 CPU",
            "Note": "Created at cpu.462",
            "Unique ID": "rdCR.j8NaKXDZtZ6",
            "Hardware Class": "cpu",
            "Arch": "X86-64",
            "Vendor": "GenuineIntel",
            "Model": '6.6.3 "QEMU Virtual CPU version 2.5+"',
            "Features": [
                "fpu",
                "de",
                "pse",
                "tsc",
                "msr",
                "pae",
                "mce",
                "cx8",
                "apic",
                "sep",
                "mtrr",
                "pge",
                "mca",
                "cmov",
                "pse36",
                "clflush",
                "mmx",
                "fxsr",
                "sse",
                "sse2",
                "syscall",
                "nx",
                "lm",
                "rep_good",
                "nopl",
                "xtopology",
                "cpuid",
                "tsc_known_freq",
                "pni",
                "cx16",
                "x2apic",
                "hypervisor",
                "lahf_lm",
                "cpuid_fault",
                "pti",
            ],
            "Clock": "3591 MHz",
            "BogoMips": "7182.68",
            "Cache": "16384 kb",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
    }


def test__hwinfo_parse_full_nic():
    hwinfo = textwrap.dedent(
        """
        28: None 00.0: 10700 Loopback
          [Created at net.126]
          Unique ID: ZsBS.GQNx7L4uPNA
          SysFS ID: /class/net/lo
          Hardware Class: network interface
          Model: "Loopback network interface"
          Device File: lo
          Link detected: yes
          Config Status: cfg=new, avail=yes, need=no, active=unknown

        29: None 03.0: 10701 Ethernet
          [Created at net.126]
          Unique ID: U2Mp.ndpeucax6V1
          Parent ID: vWuh.VIRhsc57kTD
          SysFS ID: /class/net/ens3
          SysFS Device Link: /devices/pci0000:00/0000:00:03.0/virtio0
          Hardware Class: network interface
          Model: "Ethernet network interface"
          Driver: "virtio_net"
          Driver Modules: "virtio_net"
          Device File: ens3
          HW Address: 52:54:00:12:34:56
          Permanent HW Address: 52:54:00:12:34:56
          Link detected: yes
          Config Status: cfg=new, avail=yes, need=no, active=unknown
          Attached to: #19 (Ethernet controller)
    """
    )
    assert devinfo._hwinfo_parse_full(hwinfo) == {
        "28": {
            "None 00.0": "10700 Loopback",
            "Note": "Created at net.126",
            "Unique ID": "ZsBS.GQNx7L4uPNA",
            "SysFS ID": "/class/net/lo",
            "Hardware Class": "network interface",
            "Model": "Loopback network interface",
            "Device File": "lo",
            "Link detected": "yes",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
        },
        "29": {
            "None 03.0": "10701 Ethernet",
            "Note": "Created at net.126",
            "Unique ID": "U2Mp.ndpeucax6V1",
            "Parent ID": "vWuh.VIRhsc57kTD",
            "SysFS ID": "/class/net/ens3",
            "SysFS Device Link": "/devices/pci0000:00/0000:00:03.0/virtio0",
            "Hardware Class": "network interface",
            "Model": "Ethernet network interface",
            "Driver": ["virtio_net"],
            "Driver Modules": ["virtio_net"],
            "Device File": "ens3",
            "HW Address": "52:54:00:12:34:56",
            "Permanent HW Address": "52:54:00:12:34:56",
            "Link detected": "yes",
            "Config Status": {
                "cfg": "new",
                "avail": "yes",
                "need": "no",
                "active": "unknown",
            },
            "Attached to": {"Handle": "#19 (Ethernet controller)"},
        },
    }



import pytest

import salt.grains.core as core
from tests.support.mock import MagicMock, Mock, patch

pytestmark = [
    pytest.mark.skip_unless_on_windows,
]

wmi = pytest.importorskip("wmi", reason="WMI only available on Windows")


def test__windows_platform_data_index_errors():
    # mock = [MagicMock(Manufacturer="Dell Inc.", Model="Precision 5820 Tower")]
    # mock = [MagicMock(
    #     Version="10.0.22631",
    #     Caption="Microsoft Windows 11 Enterprise",
    #     Manufacturer="Microsoft Corporation",
    #     ProductType=1,
    # )]

    WMI = Mock()
    platform_version = "1.2.3"
    os_version_info = {"Domain": "test", "DomainType": "test_type"}

    with patch("salt.utils.winapi.Com", MagicMock()), patch.object(
        wmi, "WMI", Mock(return_value=WMI)
    ), patch.object(WMI, "Win32_ComputerSystem", return_value=[]), patch.object(
        WMI, "Win32_OperatingSystem", return_value=[]
    ), patch.object(
        WMI, "Win32_BIOS", return_value=[]
    ), patch.object(
        WMI, "Win32_TimeZone", return_value=[]
    ), patch.object(
        WMI, "Win32_ComputerSystemProduct", return_value=[]
    ), patch.object(
        WMI, "Win32_BaseBoard", return_value=[]
    ), patch(
        "platform.version", return_value=platform_version
    ), patch(
        "salt.utils.win_osinfo.get_join_info", return_value=os_version_info
    ):
        result = core._windows_platform_data()
        expected = {
            "biosstring": None,
            "biosversion": None,
            "kernelrelease": None,
            "kernelversion": platform_version,
            "manufacturer": None,
            "motherboard": {"productname": None, "serialnumber": None},
            "osfullname": None,
            "osmanufacturer": None,
            "osrelease": None,
            "osservicepack": None,
            "osversion": None,
            "productname": None,
            "serialnumber": None,
            "timezone": None,
            "uuid": None,
            "windowsdomain": os_version_info["Domain"],
            "windowsdomaintype": os_version_info["DomainType"],
        }
        assert result == expected


def test__windows_platform_data_computer_system():
    mock = [MagicMock(Manufacturer="Dell Inc.", Model="Precision 5820 Tower")]
    WMI = Mock()
    platform_version = "1.2.3"
    os_version_info = {"Domain": "test", "DomainType": "test_type"}
    with patch("salt.utils.winapi.Com", MagicMock()), patch.object(
        wmi, "WMI", Mock(return_value=WMI)
    ), patch.object(WMI, "Win32_ComputerSystem", return_value=mock), patch.object(
        WMI, "Win32_OperatingSystem", return_value=[]
    ), patch.object(
        WMI, "Win32_BIOS", return_value=[]
    ), patch.object(
        WMI, "Win32_TimeZone", return_value=[]
    ), patch.object(
        WMI, "Win32_ComputerSystemProduct", return_value=[]
    ), patch.object(
        WMI, "Win32_BaseBoard", return_value=[]
    ), patch(
        "platform.version", return_value=platform_version
    ), patch(
        "salt.utils.win_osinfo.get_join_info", return_value=os_version_info
    ):
        result = core._windows_platform_data()
        assert result["manufacturer"] == "Dell Inc."
        assert result["productname"] == "Precision 5820 Tower"


def test__windows_platform_data_operating_system():
    mock = [
        MagicMock(
            Version="10.0.22631",
            Caption="Microsoft Windows 11 Enterprise",
            Manufacturer="Microsoft Corporation",
            ProductType=1,
        )
    ]

    WMI = Mock()
    platform_version = "1.2.3"
    os_version_info = {"Domain": "test", "DomainType": "test_type"}
    with patch("salt.utils.winapi.Com", MagicMock()), patch.object(
        wmi, "WMI", Mock(return_value=WMI)
    ), patch.object(WMI, "Win32_ComputerSystem", return_value=[]), patch.object(
        WMI, "Win32_OperatingSystem", return_value=mock
    ), patch.object(
        WMI, "Win32_BIOS", return_value=[]
    ), patch.object(
        WMI, "Win32_TimeZone", return_value=[]
    ), patch.object(
        WMI, "Win32_ComputerSystemProduct", return_value=[]
    ), patch.object(
        WMI, "Win32_BaseBoard", return_value=[]
    ), patch(
        "platform.version", return_value=platform_version
    ), patch(
        "salt.utils.win_osinfo.get_join_info", return_value=os_version_info
    ):
        result = core._windows_platform_data()
        assert result["kernelrelease"] == "10.0.22631"
        assert result["osfullname"] == "Microsoft Windows 11 Enterprise"
        assert result["osmanufacturer"] == "Microsoft Corporation"
        assert result["osrelease"] == "11"
        assert result["osversion"] == "10.0.22631"


def test__windows_platform_data_bios():
    mock = [
        MagicMock(
            Name="11.22.33",
            Version="DELL   - 1072009",
            SerialNumber="BCF3H13",
        )
    ]

    WMI = Mock()
    platform_version = "1.2.3"
    os_version_info = {"Domain": "test", "DomainType": "test_type"}
    with patch("salt.utils.winapi.Com", MagicMock()), patch.object(
        wmi, "WMI", Mock(return_value=WMI)
    ), patch.object(WMI, "Win32_ComputerSystem", return_value=[]), patch.object(
        WMI, "Win32_OperatingSystem", return_value=[]
    ), patch.object(
        WMI, "Win32_BIOS", return_value=mock
    ), patch.object(
        WMI, "Win32_TimeZone", return_value=[]
    ), patch.object(
        WMI, "Win32_ComputerSystemProduct", return_value=[]
    ), patch.object(
        WMI, "Win32_BaseBoard", return_value=[]
    ), patch(
        "platform.version", return_value=platform_version
    ), patch(
        "salt.utils.win_osinfo.get_join_info", return_value=os_version_info
    ):
        result = core._windows_platform_data()
        assert result["biosversion"] == "11.22.33"
        assert result["biosstring"] == "DELL   - 1072009"
        assert result["serialnumber"] == "BCF3H13"


def test__windows_platform_data_timezone():
    mock = [
        MagicMock(
            Description="(UTC-07:00) Mountain Time (US & Canada)",
        )
    ]

    WMI = Mock()
    platform_version = "1.2.3"
    os_version_info = {"Domain": "test", "DomainType": "test_type"}
    with patch("salt.utils.winapi.Com", MagicMock()), patch.object(
        wmi, "WMI", Mock(return_value=WMI)
    ), patch.object(WMI, "Win32_ComputerSystem", return_value=[]), patch.object(
        WMI, "Win32_OperatingSystem", return_value=[]
    ), patch.object(
        WMI, "Win32_BIOS", return_value=[]
    ), patch.object(
        WMI, "Win32_TimeZone", return_value=mock
    ), patch.object(
        WMI, "Win32_ComputerSystemProduct", return_value=[]
    ), patch.object(
        WMI, "Win32_BaseBoard", return_value=[]
    ), patch(
        "platform.version", return_value=platform_version
    ), patch(
        "salt.utils.win_osinfo.get_join_info", return_value=os_version_info
    ):
        result = core._windows_platform_data()
        assert result["timezone"] == "(UTC-07:00) Mountain Time (US & Canada)"


def test__windows_platform_data_computer_system_product():
    mock = [
        MagicMock(
            UUID="4C4C4544-0043-4610-8030-C2C04F483033",
        )
    ]

    WMI = Mock()
    platform_version = "1.2.3"
    os_version_info = {"Domain": "test", "DomainType": "test_type"}
    with patch("salt.utils.winapi.Com", MagicMock()), patch.object(
        wmi, "WMI", Mock(return_value=WMI)
    ), patch.object(WMI, "Win32_ComputerSystem", return_value=[]), patch.object(
        WMI, "Win32_OperatingSystem", return_value=[]
    ), patch.object(
        WMI, "Win32_BIOS", return_value=[]
    ), patch.object(
        WMI, "Win32_TimeZone", return_value=[]
    ), patch.object(
        WMI, "Win32_ComputerSystemProduct", return_value=mock
    ), patch.object(
        WMI, "Win32_BaseBoard", return_value=[]
    ), patch(
        "platform.version", return_value=platform_version
    ), patch(
        "salt.utils.win_osinfo.get_join_info", return_value=os_version_info
    ):
        result = core._windows_platform_data()
        assert result["uuid"] == "4c4c4544-0043-4610-8030-c2c04f483033"


def test__windows_platform_data_baseboard():
    mock = [
        MagicMock(
            Product="002KVM",
            SerialNumber="/BCF0H03/CNFCW0097F00TM/",
        )
    ]

    WMI = Mock()
    platform_version = "1.2.3"
    os_version_info = {"Domain": "test", "DomainType": "test_type"}
    with patch("salt.utils.winapi.Com", MagicMock()), patch.object(
        wmi, "WMI", Mock(return_value=WMI)
    ), patch.object(WMI, "Win32_ComputerSystem", return_value=[]), patch.object(
        WMI, "Win32_OperatingSystem", return_value=[]
    ), patch.object(
        WMI, "Win32_BIOS", return_value=[]
    ), patch.object(
        WMI, "Win32_TimeZone", return_value=[]
    ), patch.object(
        WMI, "Win32_ComputerSystemProduct", return_value=[]
    ), patch.object(
        WMI, "Win32_BaseBoard", return_value=mock
    ), patch(
        "platform.version", return_value=platform_version
    ), patch(
        "salt.utils.win_osinfo.get_join_info", return_value=os_version_info
    ):
        result = core._windows_platform_data()
        assert result["motherboard"]["productname"] == "002KVM"
        assert result["motherboard"]["serialnumber"] == "/BCF0H03/CNFCW0097F00TM/"



import pytest

import salt.utils.json
from salt.netapi.rest_tornado import saltnado

# TODO: run all the same tests from the root handler, but for now since they are
# the same code, we'll just sanity check


@pytest.fixture
def app_urls():
    return [
        ("/run", saltnado.RunSaltAPIHandler),
    ]


@pytest.mark.slow_test
async def test_get(http_client, salt_minion, salt_sub_minion):
    low = [{"client": "local", "tgt": "*", "fun": "test.ping"}]
    response = await http_client.fetch(
        "/run",
        method="POST",
        body=salt.utils.json.dumps(low),
    )
    response_obj = salt.utils.json.loads(response.body)
    ret = response_obj["return"]
    assert sorted(ret[0]) == sorted([salt_minion.id, salt_sub_minion.id])



import datetime
import hashlib
import logging
import os
import pprint
import textwrap

import pytest

import salt.utils.files
from tests.support.case import ModuleCase
from tests.support.helpers import with_tempfile
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS

try:
    import M2Crypto  # pylint: disable=W0611

    HAS_M2CRYPTO = True
except ImportError:
    HAS_M2CRYPTO = False

log = logging.getLogger(__name__)


@pytest.mark.usefixtures("salt_sub_minion")
@pytest.mark.skipif(not HAS_M2CRYPTO, reason="Skip when no M2Crypto found")
class x509Test(ModuleCase, SaltReturnAssertsMixin):
    @classmethod
    def setUpClass(cls):
        cert_path = os.path.join(RUNTIME_VARS.BASE_FILES, "x509_test.crt")
        with salt.utils.files.fopen(cert_path) as fp:
            cls.x509_cert_text = fp.read()

    def setUp(self):
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "signing_policies.sls"), "w"
        ) as fp:
            fp.write(
                textwrap.dedent(
                    """\
                x509_signing_policies:
                  ca_policy:
                    - minions: '*'
                    - signing_private_key: {0}/pki/ca.key
                    - signing_cert: {0}/pki/ca.crt
                    - O: Test Company
                    - basicConstraints: "CA:false"
                    - keyUsage: "critical digitalSignature, keyEncipherment"
                    - extendedKeyUsage: "critical serverAuth, clientAuth"
                    - subjectKeyIdentifier: hash
                    - authorityKeyIdentifier: keyid
                    - days_valid: 730
                    - copypath: {0}/pki
                  compound_match:
                    - minions: 'G@x509_test_grain:correct_value'
                    - signing_private_key: {0}/pki/ca.key
                    - signing_cert: {0}/pki/ca.crt
                    - O: Test Company
                    - basicConstraints: "CA:false"
                    - keyUsage: "critical digitalSignature, keyEncipherment"
                    - extendedKeyUsage: "critical serverAuth, clientAuth"
                    - subjectKeyIdentifier: hash
                    - authorityKeyIdentifier: keyid
                    - days_valid: 730
                    - copypath: {0}/pki
                     """.format(
                        RUNTIME_VARS.TMP
                    )
                )
            )
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "top.sls"), "w"
        ) as fp:
            fp.write(
                textwrap.dedent(
                    """\
                     base:
                       '*':
                         - signing_policies
                     """
                )
            )
        self.run_function("saltutil.refresh_pillar")
        self.run_function(
            "grains.set", ["x509_test_grain", "correct_value"], minion_tgt="sub_minion"
        )
        self.run_function(
            "grains.set", ["x509_test_grain", "not_correct_value"], minion_tgt="minion"
        )

    def tearDown(self):
        os.remove(os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "signing_policies.sls"))
        os.remove(os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "top.sls"))
        certs_path = os.path.join(RUNTIME_VARS.TMP, "pki")
        if os.path.exists(certs_path):
            salt.utils.files.rm_rf(certs_path)
        self.run_function("saltutil.refresh_pillar")
        self.run_function("grains.delkey", ["x509_test_grain"], minion_tgt="sub_minion")
        self.run_function("grains.delkey", ["x509_test_grain"], minion_tgt="minion")

    def run_function(self, *args, **kwargs):  # pylint: disable=arguments-differ
        ret = super().run_function(*args, **kwargs)
        return ret

    @staticmethod
    def file_checksum(path):
        hash = hashlib.sha1()
        with salt.utils.files.fopen(path, "rb") as f:
            for block in iter(lambda: f.read(4096), b""):
                hash.update(block)
        return hash.hexdigest()

    @with_tempfile(suffix=".pem", create=False)
    @pytest.mark.slow_test
    def test_issue_49027(self, pemfile):
        ret = self.run_state("x509.pem_managed", name=pemfile, text=self.x509_cert_text)
        assert isinstance(ret, dict), ret
        ret = ret[next(iter(ret))]
        assert ret.get("result") is True, ret
        with salt.utils.files.fopen(pemfile) as fp:
            result = fp.readlines()
        self.assertEqual(self.x509_cert_text.splitlines(True), result)

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    @pytest.mark.slow_test
    def test_issue_49008(self, keyfile, crtfile):
        ret = self.run_function(
            "state.apply",
            ["issue-49008"],
            pillar={"keyfile": keyfile, "crtfile": crtfile},
        )
        assert isinstance(ret, dict), ret
        for state_result in ret.values():
            assert state_result["result"] is True, state_result
        assert os.path.exists(keyfile)
        assert os.path.exists(crtfile)

    @pytest.mark.slow_test
    def test_cert_signing(self):
        ret = self.run_function(
            "state.apply", ["x509.cert_signing"], pillar={"tmp_dir": RUNTIME_VARS.TMP}
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]

    @pytest.mark.slow_test
    def test_cert_signing_based_on_csr(self):
        ret = self.run_function(
            "state.apply",
            ["x509.cert_signing_based_on_csr"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]

    @pytest.mark.slow_test
    def test_proper_cert_comparison(self):
        # In this SLS we define two certs which have identical content.
        # The first one is expected to be created.
        # The second one is expected to be recognized as already present.
        ret = self.run_function(
            "state.apply",
            ["x509.proper_cert_comparison"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )
        # check the first generated cert
        first_key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert first_key in ret
        assert "changes" in ret[first_key]
        assert "Certificate" in ret[first_key]["changes"]
        assert "New" in ret[first_key]["changes"]["Certificate"]
        # check whether the second defined cert is considered to match the first one
        second_key = (
            "x509_|-second_test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
                RUNTIME_VARS.TMP
            )
        )
        assert second_key in ret
        assert "changes" in ret[second_key]
        assert ret[second_key]["changes"] == {}

    @pytest.mark.slow_test
    def test_crl_managed(self):
        ret = self.run_function(
            "state.apply", ["x509.crl_managed"], pillar={"tmp_dir": RUNTIME_VARS.TMP}
        )
        key = "x509_|-{}/pki/ca.crl_|-{}/pki/ca.crl_|-crl_managed".format(
            RUNTIME_VARS.TMP, RUNTIME_VARS.TMP
        )

        # hints for easier debugging
        # import json
        # print(json.dumps(ret[key], indent=4, sort_keys=True))
        # print(ret[key]['comment'])

        assert key in ret
        assert "changes" in ret[key]
        self.assertEqual(ret[key]["result"], True)
        assert "New" in ret[key]["changes"]
        assert "Revoked Certificates" in ret[key]["changes"]["New"]
        self.assertEqual(
            ret[key]["changes"]["Old"],
            f"{RUNTIME_VARS.TMP}/pki/ca.crl does not exist.",
        )

    @pytest.mark.slow_test
    def test_crl_managed_replacing_existing_crl(self):
        os.mkdir(os.path.join(RUNTIME_VARS.TMP, "pki"))
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP, "pki/ca.crl"), "wb"
        ) as crl_file:
            crl_file.write(
                b"""-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCjdjbgL4kQ8Lu73xeRRM1q3C3K3ptfCLpyfw38LRnymxaoJ6ls
pNSx2dU1uJ89YKFlYLo1QcEk4rJ2fdIjarV0kuNCY3rC8jYUp9BpAU5Z6p9HKeT1
2rTPH81JyjbQDR5PyfCyzYOQtpwpB4zIUUK/Go7tTm409xGKbbUFugJNgQIDAQAB
AoGAF24we34U1ZrMLifSRv5nu3OIFNZHyx2DLDpOFOGaII5edwgIXwxZeIzS5Ppr
yO568/8jcdLVDqZ4EkgCwRTgoXRq3a1GLHGFmBdDNvWjSTTMLoozuM0t2zjRmIsH
hUd7tnai9Lf1Bp5HlBEhBU2gZWk+SXqLvxXe74/+BDAj7gECQQDRw1OPsrgTvs3R
3MNwX6W8+iBYMTGjn6f/6rvEzUs/k6rwJluV7n8ISNUIAxoPy5g5vEYK6Ln/Ttc7
u0K1KNlRAkEAx34qcxjuswavL3biNGE+8LpDJnJx1jaNWoH+ObuzYCCVMusdT2gy
kKuq9ytTDgXd2qwZpIDNmscvReFy10glMQJAXebMz3U4Bk7SIHJtYy7OKQzn0dMj
35WnRV81c2Jbnzhhu2PQeAvt/i1sgEuzLQL9QEtSJ6wLJ4mJvImV0TdaIQJAAYyk
TcKK0A8kOy0kMp3yvDHmJZ1L7wr7bBGIZPBlQ0Ddh8i1sJExm1gJ+uN2QKyg/XrK
tDFf52zWnCdVGgDwcQJALW/WcbSEK+JVV6KDJYpwCzWpKIKpBI0F6fdCr1G7Xcwj
c9bcgp7D7xD+TxWWNj4CSXEccJgGr91StV+gFg4ARQ==
-----END RSA PRIVATE KEY-----
"""
            )

        ret = self.run_function(
            "state.apply", ["x509.crl_managed"], pillar={"tmp_dir": RUNTIME_VARS.TMP}
        )
        key = "x509_|-{}/pki/ca.crl_|-{}/pki/ca.crl_|-crl_managed".format(
            RUNTIME_VARS.TMP, RUNTIME_VARS.TMP
        )

        # hints for easier debugging
        # import json
        # print(json.dumps(ret[key], indent=4, sort_keys=True))
        # print(ret[key]['comment'])

        assert key in ret
        assert "changes" in ret[key]
        self.assertEqual(ret[key]["result"], True)
        assert "New" in ret[key]["changes"]
        assert "Revoked Certificates" in ret[key]["changes"]["New"]
        self.assertEqual(
            ret[key]["changes"]["Old"],
            f"{RUNTIME_VARS.TMP}/pki/ca.crl is not a valid CRL.",
        )

    def test_cert_issue_not_before_not_after(self):
        ret = self.run_function(
            "state.apply",
            ["test_cert_not_before_not_after"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]
        assert "Not Before" in ret[key]["changes"]["Certificate"]["New"]
        assert "Not After" in ret[key]["changes"]["Certificate"]["New"]
        not_before = ret[key]["changes"]["Certificate"]["New"]["Not Before"]
        not_after = ret[key]["changes"]["Certificate"]["New"]["Not After"]
        assert not_before == "2019-05-05 00:00:00"
        assert not_after == "2020-05-05 14:30:00"

    def test_cert_issue_not_before(self):
        ret = self.run_function(
            "state.apply",
            ["test_cert_not_before"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]
        assert "Not Before" in ret[key]["changes"]["Certificate"]["New"]
        assert "Not After" in ret[key]["changes"]["Certificate"]["New"]
        not_before = ret[key]["changes"]["Certificate"]["New"]["Not Before"]
        assert not_before == "2019-05-05 00:00:00"

    def test_cert_issue_not_after(self):
        ret = self.run_function(
            "state.apply", ["test_cert_not_after"], pillar={"tmp_dir": RUNTIME_VARS.TMP}
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]
        assert "Not Before" in ret[key]["changes"]["Certificate"]["New"]
        assert "Not After" in ret[key]["changes"]["Certificate"]["New"]
        not_after = ret[key]["changes"]["Certificate"]["New"]["Not After"]
        assert not_after == "2020-05-05 14:30:00"

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_issue_41858(self, keyfile, crtfile):
        ret_key = f"x509_|-test_crt_|-{crtfile}_|-certificate_managed"
        signing_policy = "no_such_policy"
        ret = self.run_function(
            "state.apply",
            ["issue-41858.gen_cert"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "tmp_dir": RUNTIME_VARS.TMP,
            },
        )
        self.assertTrue(ret[ret_key]["result"])
        cert_sum = self.file_checksum(crtfile)

        ret = self.run_function(
            "state.apply",
            ["issue-41858.check"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "signing_policy": signing_policy,
            },
        )
        self.assertFalse(ret[ret_key]["result"])
        # self.assertSaltCommentRegexpMatches(ret[ret_key], "Signing policy {0} does not exist".format(signing_policy))
        self.assertEqual(self.file_checksum(crtfile), cert_sum)

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_compound_match_minion_have_correct_grain_value(self, keyfile, crtfile):
        ret_key = f"x509_|-test_crt_|-{crtfile}_|-certificate_managed"
        signing_policy = "compound_match"
        ret = self.run_function(
            "state.apply",
            ["x509_compound_match.gen_ca"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )

        # sub_minion have grain set and CA is on other minion
        # CA minion have same grain with incorrect value
        ret = self.run_function(
            "state.apply",
            ["x509_compound_match.check"],
            minion_tgt="sub_minion",
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "signing_policy": signing_policy,
            },
        )
        self.assertTrue(ret[ret_key]["result"])

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_compound_match_ca_have_correct_grain_value(self, keyfile, crtfile):
        self.run_function(
            "grains.set", ["x509_test_grain", "correct_value"], minion_tgt="minion"
        )
        self.run_function(
            "grains.set",
            ["x509_test_grain", "not_correct_value"],
            minion_tgt="sub_minion",
        )

        ret_key = f"x509_|-test_crt_|-{crtfile}_|-certificate_managed"
        signing_policy = "compound_match"
        self.run_function(
            "state.apply",
            ["x509_compound_match.gen_ca"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )

        ret = self.run_function(
            "state.apply",
            ["x509_compound_match.check"],
            minion_tgt="sub_minion",
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "signing_policy": signing_policy,
            },
        )
        self.assertFalse(ret[ret_key]["result"])

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_self_signed_cert(self, keyfile, crtfile):
        """
        Self-signed certificate, no CA.
        Run the state twice to confirm the cert is only created once
        and its contents don't change.
        """
        first_run = self.run_function(
            "state.apply",
            ["x509.self_signed"],
            pillar={"keyfile": keyfile, "crtfile": crtfile},
        )
        key = f"x509_|-self_signed_cert_|-{crtfile}_|-certificate_managed"
        self.assertIn("New", first_run[key]["changes"]["Certificate"])
        self.assertEqual(
            "Certificate is valid and up to date",
            first_run[key]["changes"]["Status"]["New"],
        )
        self.assertTrue(os.path.exists(crtfile), "Certificate was not created.")

        with salt.utils.files.fopen(crtfile, "r") as first_cert:
            cert_contents = first_cert.read()

        second_run = self.run_function(
            "state.apply",
            ["x509.self_signed"],
            pillar={"keyfile": keyfile, "crtfile": crtfile},
        )
        self.assertEqual({}, second_run[key]["changes"])
        with salt.utils.files.fopen(crtfile, "r") as second_cert:
            self.assertEqual(
                cert_contents,
                second_cert.read(),
                "Certificate contents should not have changed.",
            )

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_old_self_signed_cert_is_recreated(self, keyfile, crtfile):
        """
        Self-signed certificate, no CA.
        First create a cert that expires in 30 days, then recreate
        the cert because the second state run requires days_remaining
        to be at least 90.
        """
        first_run = self.run_function(
            "state.apply",
            ["x509.self_signed_expiry"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "days_valid": 30,
                "days_remaining": 10,
            },
        )
        key = f"x509_|-self_signed_cert_|-{crtfile}_|-certificate_managed"
        self.assertEqual(
            "Certificate is valid and up to date",
            first_run[key]["changes"]["Status"]["New"],
        )
        expiry = datetime.datetime.strptime(
            first_run[key]["changes"]["Certificate"]["New"]["Not After"],
            "%Y-%m-%d %H:%M:%S",
        )
        self.assertEqual(29, (expiry - datetime.datetime.now()).days)
        self.assertTrue(os.path.exists(crtfile), "Certificate was not created.")

        with salt.utils.files.fopen(crtfile, "r") as first_cert:
            cert_contents = first_cert.read()

        second_run = self.run_function(
            "state.apply",
            ["x509.self_signed_expiry"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "days_valid": 180,
                "days_remaining": 90,
            },
        )
        self.assertEqual(
            "Certificate needs renewal: 29 days remaining but it needs to be at"
            " least 90",
            second_run[key]["changes"]["Status"]["Old"],
        )
        expiry = datetime.datetime.strptime(
            second_run[key]["changes"]["Certificate"]["New"]["Not After"],
            "%Y-%m-%d %H:%M:%S",
        )
        self.assertEqual(179, (expiry - datetime.datetime.now()).days)
        with salt.utils.files.fopen(crtfile, "r") as second_cert:
            self.assertNotEqual(
                cert_contents,
                second_cert.read(),
                "Certificate contents should have changed.",
            )

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_mismatched_self_signed_cert_is_recreated(self, keyfile, crtfile):
        """
        Self-signed certificate, no CA.
        First create a cert, then run the state again with a different
        subjectAltName. The cert should be recreated.
        Finally, run once more with the same subjectAltName as the
        second run. Nothing should change.
        """
        first_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "subjectAltName": "DNS:alt.service.local",
            },
        )
        key = f"x509_|-self_signed_cert_|-{crtfile}_|-certificate_managed"
        self.assertEqual(
            "Certificate is valid and up to date",
            first_run[key]["changes"]["Status"]["New"],
        )
        sans = first_run[key]["changes"]["Certificate"]["New"]["X509v3 Extensions"][
            "subjectAltName"
        ]
        self.assertEqual("DNS:alt.service.local", sans)
        self.assertTrue(os.path.exists(crtfile), "Certificate was not created.")

        with salt.utils.files.fopen(crtfile, "r") as first_cert:
            first_cert_contents = first_cert.read()

        second_run_pillar = {
            "keyfile": keyfile,
            "crtfile": crtfile,
            "subjectAltName": "DNS:alt1.service.local, DNS:alt2.service.local",
        }
        second_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar=second_run_pillar,
        )
        self.assertEqual(
            "Certificate properties are different: X509v3 Extensions",
            second_run[key]["changes"]["Status"]["Old"],
        )
        sans = second_run[key]["changes"]["Certificate"]["New"]["X509v3 Extensions"][
            "subjectAltName"
        ]
        self.assertEqual("DNS:alt1.service.local, DNS:alt2.service.local", sans)
        with salt.utils.files.fopen(crtfile, "r") as second_cert:
            second_cert_contents = second_cert.read()
            self.assertNotEqual(
                first_cert_contents,
                second_cert_contents,
                "Certificate contents should have changed.",
            )

        third_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar=second_run_pillar,
        )
        self.assertEqual({}, third_run[key]["changes"])
        with salt.utils.files.fopen(crtfile, "r") as third_cert:
            self.assertEqual(
                second_cert_contents,
                third_cert.read(),
                "Certificate contents should not have changed.",
            )

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_certificate_managed_with_managed_private_key_does_not_error(
        self, keyfile, crtfile
    ):
        """
        Test using the deprecated managed_private_key arg in certificate_managed does not throw an error.

        TODO: Remove this test in Aluminium when the arg is removed.
        """
        self.run_state("x509.private_key_managed", name=keyfile, bits=4096)
        ret = self.run_state(
            "x509.certificate_managed",
            name=crtfile,
            CN="localhost",
            signing_private_key=keyfile,
            managed_private_key={"name": keyfile, "bits": 4096},
        )
        key = "x509_|-{0}_|-{0}_|-certificate_managed".format(crtfile)
        self.assertEqual(True, ret[key]["result"])

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_file_properties_are_updated(self, keyfile, crtfile):
        """
        Self-signed certificate, no CA.
        First create a cert, then run the state again with different
        file mode. The cert should not be recreated, but the file
        should be updated.
        Finally, run once more with the same file mode as the second
        run. Nothing should change.
        """
        first_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar={"keyfile": keyfile, "crtfile": crtfile, "fileMode": "0755"},
        )
        key = f"x509_|-self_signed_cert_|-{crtfile}_|-certificate_managed"
        self.assertEqual(
            "Certificate is valid and up to date",
            first_run[key]["changes"]["Status"]["New"],
        )
        self.assertTrue(os.path.exists(crtfile), "Certificate was not created.")
        self.assertEqual("0755", oct(os.stat(crtfile).st_mode)[-4:])

        second_run_pillar = {
            "keyfile": keyfile,
            "crtfile": crtfile,
            "mode": "0600",
        }
        second_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar=second_run_pillar,
        )
        self.assertEqual("0600", oct(os.stat(crtfile).st_mode)[-4:])

        third_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar=second_run_pillar,
        )
        self.assertEqual({}, third_run[key]["changes"])
        self.assertEqual("0600", oct(os.stat(crtfile).st_mode)[-4:])

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_file_managed_failure(self, keyfile, crtfile):
        """
        Test that a failure in the file.managed call marks the state
        call as failed.
        """
        crtfile_pieces = os.path.split(crtfile)
        bad_crtfile = os.path.join(
            crtfile_pieces[0], "deeply/nested", crtfile_pieces[1]
        )
        ret = self.run_function(
            "state.apply",
            ["x509.self_signed_file_error"],
            pillar={"keyfile": keyfile, "crtfile": bad_crtfile},
        )

        key = f"x509_|-self_signed_cert_|-{bad_crtfile}_|-certificate_managed"
        self.assertFalse(ret[key]["result"], "State should have failed.")
        self.assertEqual({}, ret[key]["changes"])
        self.assertFalse(
            os.path.exists(crtfile), "Certificate should not have been created."
        )

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_py2_generated_cert_is_not_recreated(self, keyfile, crtfile):
        keyfile_contents = textwrap.dedent(
            """\
        -----BEGIN RSA PRIVATE KEY-----
        MIIEpAIBAAKCAQEAp5PQyx5NlYrfzd7vU/Xb2YR5qbWWtpWWoKmJC1gML5v5DBI7
        +p/kAHNNmK8uqHXTaI4N/zgarfjrg4zceq2Du7pP0xiCAYolhFqF78ibxNrN4OkT
        UPm2kM88iJ8Z14Yph8ueSxLIlujCGaEFhr6wRzTj4T9b+0Bb/PZHI2t5YwtIooVM
        EFCBFkt4bb004tO0D9q0CPPVT2AsGmxnY43Aj3Epy++kqmaWj1hIucSprkDrAXFS
        WacBQPFQ8XctnL2Z1Q6CJ5WUNrW8ohAJ9RJkwjiqbZTwYIPSSrl+FO3XqDY70SxU
        3xDeqhU4zvyjxJ8w9SPqTUu/C3BZtRBT9dCBEQIDAQABAoIBAQCZvS23u1RYVrEe
        sWGF+LA67aOkg9kCJ1iqiv8UrjF32DNy1KO8OcY2d5H/+u/mUzqh2HmU5QbtBsoi
        xS9dSSTrLHGhbAGRogjrVRU9uCDYSBjLN2mmR4IrdkTF3pkZtpcRY0gU/eWTNXUl
        iCmGxhj5KtfJxZQAfLon6FW5dBdIOgxSCJhvRq0zFpWJZFGWWkBExDfeNg//0fCU
        UbjRjGacP/+R6FSJa6tevzgR7tIIapm1dY/ofPXIXsZGo1R87fRgLI1D+e84Jdds
        /U0bKzPOgAjcC1b262lJ8058pjG/nqWC0YUfpIJUVv2ciJpH3Ha+90526InLAUXA
        RWe1Z2YxAoGBANqACEKvUbxENu+XxQj0SI1co4SRTOvgbrSQGL61rDY6PvY/bOqC
        JeR0KC3MN6e7fx52tsl/eqP9iyExUpO9b0BCnGg967MivJXWUxhUdOL/r2ceQBqD
        DiPVZCFsjeNdSNihnNctAig9Po3GEUWE0ikHr3NcD+wXTnhnIEjJ/fltAoGBAMRW
        dIcOiuDLm/oDLNCpwEO4m63ymbUgeOj2cZhKMTqFmspnKnuCU1U/A8cuQcs1gydL
        7MzxVP7MZDIEqT5gGj3eyuVMAmKbvLFR2NctDIDjaUs6oz0J9NGByPNjXaYr4uMd
        EZrxD8gLZ/G+/7eKsCgBA9ksSydDo00Vf/qAsmO1AoGBANWqc+l59eyrrCj5egU6
        lKQf3gsp51WV/8v0SS5dC41vwdgdx80+/fz8FbpLRHVypWlN34sFbRFmQ6Juz/iH
        O35UZQyO2KkxI8dGcbWOCUtditHExBzo4W/rIWKJ++pFc5Hb4DqO2dgto7kR4hvg
        OX9D869UbIGLfQHCntBvLju1AoGAHpcl0sEmTD4NEFgcTGqWZTbHMsQAxOLJU+rJ
        6iNtJiQY6P5H9TRqDXci/I6te57bz2yZ+ZiEWKq51b06LVjF3evviuhb2sdPEAWj
        lmsTbqWAC1OYiXMarOXezGUn+zMNR7uIua5jehSk3lqW9x7psWHvGpA3KWf1cpYt
        +XbB1J0CgYBCSjALTv4dcn+CtS3kqb806z8H9MSZznUwSmcgvwCR5sqwLAUk1xRn
        hEqXbC1RGee3Xqv9mXPDK2LirpdRYi9Jr9ApZkrSkeaXSd2d4cy2ujUT0c7P8JrD
        i6QXb+HaFeBuS5ulYDmo4mIbCysuTsgrLzplViUy3xUQv23M/Eh1gw==
        -----END RSA PRIVATE KEY-----
        """
        )
        crtfile_contents = textwrap.dedent(
            """\
        -----BEGIN CERTIFICATE-----
        MIIEhTCCA22gAwIBAgIIUijHgif6VJUwDQYJKoZIhvcNAQELBQAwgYIxCzAJBgNV
        BAYTAkJFMRgwFgYDVQQDDA9FeGFtcGxlIFJvb3QgQ0ExETAPBgNVBAcMCEthcGVs
        bGVuMRAwDgYDVQQIDAdBbnR3ZXJwMRAwDgYDVQQKDAdFeGFtcGxlMSIwIAYJKoZI
        hvcNAQkBFhNjZXJ0YWRtQGV4YW1wbGUub3JnMB4XDTIwMDYxNjA3Mzk1OVoXDTMw
        MDYxNDA3Mzk1OVowgYIxCzAJBgNVBAYTAkJFMRgwFgYDVQQDDA9FeGFtcGxlIFJv
        b3QgQ0ExETAPBgNVBAcMCEthcGVsbGVuMRAwDgYDVQQIDAdBbnR3ZXJwMRAwDgYD
        VQQKDAdFeGFtcGxlMSIwIAYJKoZIhvcNAQkBFhNjZXJ0YWRtQGV4YW1wbGUub3Jn
        MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAp5PQyx5NlYrfzd7vU/Xb
        2YR5qbWWtpWWoKmJC1gML5v5DBI7+p/kAHNNmK8uqHXTaI4N/zgarfjrg4zceq2D
        u7pP0xiCAYolhFqF78ibxNrN4OkTUPm2kM88iJ8Z14Yph8ueSxLIlujCGaEFhr6w
        RzTj4T9b+0Bb/PZHI2t5YwtIooVMEFCBFkt4bb004tO0D9q0CPPVT2AsGmxnY43A
        j3Epy++kqmaWj1hIucSprkDrAXFSWacBQPFQ8XctnL2Z1Q6CJ5WUNrW8ohAJ9RJk
        wjiqbZTwYIPSSrl+FO3XqDY70SxU3xDeqhU4zvyjxJ8w9SPqTUu/C3BZtRBT9dCB
        EQIDAQABo4H8MIH5MA8GA1UdEwEB/wQFMAMBAf8wDgYDVR0PAQH/BAQDAgEGMB0G
        A1UdDgQWBBTmNsYLuQTxpANgTuw7LRn1qHJsjzCBtgYDVR0jBIGuMIGrgBTmNsYL
        uQTxpANgTuw7LRn1qHJsj6GBiKSBhTCBgjELMAkGA1UEBhMCQkUxGDAWBgNVBAMM
        D0V4YW1wbGUgUm9vdCBDQTERMA8GA1UEBwwIS2FwZWxsZW4xEDAOBgNVBAgMB0Fu
        dHdlcnAxEDAOBgNVBAoMB0V4YW1wbGUxIjAgBgkqhkiG9w0BCQEWE2NlcnRhZG1A
        ZXhhbXBsZS5vcmeCCFIox4In+lSVMA0GCSqGSIb3DQEBCwUAA4IBAQBnC1/kK+xr
        Vjr5Y2YRjyjm4e8I/nTU+RX2p5K+Yth3CqWO3JuDiV/31UMtPl832n2GWSgXG2pP
        B52oeuCP4Re76jqhOmJWY3CKPji+Rs16wj199i9AAcwhSF0rpi5+Fi84HtP3q6pH
        cuzZfIPW44aJ5l4k+QvTLoWzr0XujMFcYzI45i3SJqTMs8xdIP5YLN8JXtQSPw9Z
        8/nBKbPj7WTUC9cj9Cw2bz+wTpdRF4XCsUF3Vpl9fP7SK8yvv0I85LZnWQx1eQlv
        COAM5HWxUT9bWgv18zXdYkc6VLw6ufQSxxuhLMjJxuK27Ny/F18/xYLRTVnse36d
        tPJrseUPmvIK
        -----END CERTIFICATE-----
        """
        )
        slsfile = textwrap.dedent(
            """\
        {%- set ca_key_path = '"""
            + keyfile
            + """' %}
        {%- set ca_crt_path = '"""
            + crtfile
            + """' %}

        certificate.authority::private-key:
          x509.private_key_managed:
            - name: {{ ca_key_path }}
            - backup: True

        certificate.authority::certificate:
          x509.certificate_managed:
            - name: {{ ca_crt_path }}
            - signing_private_key: {{ ca_key_path }}
            - CN: Example Root CA
            - O: Example
            - C: BE
            - ST: Antwerp
            - L: Kapellen
            - Email: certadm@example.org
            - basicConstraints: "critical CA:true"
            - keyUsage: "critical cRLSign, keyCertSign"
            - subjectKeyIdentifier: hash
            - authorityKeyIdentifier: keyid,issuer:always
            - days_valid: 3650
            - days_remaining: 0
            - backup: True
            - require:
              - x509: certificate.authority::private-key
        """
        )
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_STATE_TREE, "cert.sls"), "w"
        ) as wfh:
            wfh.write(slsfile)

        # Generate the certificate twice.
        # On the first run, no key nor cert exist.
        ret = self.run_function("state.sls", ["cert"])
        log.debug(
            "First state run ret dictionary:\n%s", pprint.pformat(list(ret.values()))
        )
        for state_run_id, state_run_details in ret.items():
            if state_run_id.endswith("private_key_managed"):
                assert state_run_details["result"]
                assert "new" in state_run_details["changes"]
            if state_run_id.endswith("certificate_managed"):
                assert state_run_details["result"]
                assert "Certificate" in state_run_details["changes"]
                assert "New" in state_run_details["changes"]["Certificate"]
                assert "Status" in state_run_details["changes"]
                assert "New" in state_run_details["changes"]["Status"]
        # On the second run, they exist and should not trigger any modification
        ret = self.run_function("state.sls", ["cert"])
        log.debug(
            "Second state run ret dictionary:\n%s", pprint.pformat(list(ret.values()))
        )
        for state_run_id, state_run_details in ret.items():
            if state_run_id.endswith("private_key_managed"):
                assert state_run_details["result"]
                assert state_run_details["changes"] == {}
            if state_run_id.endswith("certificate_managed"):
                assert state_run_details["result"]
                assert state_run_details["changes"] == {}
        # Now we repleace they key and cert contents with the contents of the above
        # call, but under Py2
        with salt.utils.files.fopen(keyfile, "w") as wfh:
            wfh.write(keyfile_contents)
        with salt.utils.files.fopen(keyfile) as rfh:
            log.debug("Written keyfile, %r, contents:\n%s", keyfile, rfh.read())
        with salt.utils.files.fopen(crtfile, "w") as wfh:
            wfh.write(crtfile_contents)
        with salt.utils.files.fopen(crtfile) as rfh:
            log.debug("Written crtfile, %r, contents:\n%s", crtfile, rfh.read())
        # We should not trigger any modification
        ret = self.run_function("state.sls", ["cert"])
        log.debug(
            "Third state run ret dictionary:\n%s", pprint.pformat(list(ret.values()))
        )
        for state_run_id, state_run_details in ret.items():
            if state_run_id.endswith("private_key_managed"):
                assert state_run_details["result"]
                assert state_run_details["changes"] == {}
            if state_run_id.endswith("certificate_managed"):
                assert state_run_details["result"]
                assert state_run_details["changes"] == {}



"""
Cluster integration tests.
"""

import salt.utils.event


def test_basic_cluster_setup(
    cluster_master_1, cluster_master_2, cluster_pki_path, cluster_cache_path
):
    cli1 = cluster_master_1.salt_run_cli(timeout=120)
    ret = cli1.run("config.get", "cluster_pki_dir")
    assert str(cluster_pki_path) == ret.stdout
    ret = cli1.run("config.get", "cache_dir")
    assert str(cluster_cache_path) == ret.stdout
    ret = cli1.run("config.get", "cluster_peers")
    ret.data.sort()
    assert ["127.0.0.2", "127.0.0.3"] == ret.data

    cli2 = cluster_master_2.salt_run_cli(timeout=120)
    ret = cli2.run("config.get", "cluster_pki_dir")
    assert str(cluster_pki_path) == ret.stdout
    ret = cli2.run("config.get", "cache_dir")
    assert str(cluster_cache_path) == ret.stdout
    ret = cli2.run("config.get", "cluster_peers")
    ret.data.sort()
    assert ["127.0.0.1", "127.0.0.3"] == ret.data

    # Check for shared keys. Note: The third master 127.0.0.3 was never
    # started.
    peers_path = cluster_pki_path / "peers"
    unexpected = False
    found = []
    for key_path in peers_path.iterdir():
        if key_path.name == "127.0.0.1.pub":
            found.append("127.0.0.1")
        elif key_path.name == "127.0.0.2.pub":
            found.append("127.0.0.2")
        else:
            unexpected = True

    found.sort()

    assert ["127.0.0.1", "127.0.0.2"] == found
    assert unexpected is False

    assert (cluster_pki_path / ".aes").exists()


def test_basic_cluster_event(cluster_master_1, cluster_master_2):
    with salt.utils.event.get_event(
        "master", opts=cluster_master_2.config, listen=True
    ) as event2:
        event1 = salt.utils.event.get_event("master", opts=cluster_master_2.config)
        data = {"meh": "bah"}
        event1.fire_event(data, "meh/bah")
        evt = event2.get_event(tag="meh/bah", wait=5)
        assert data == evt


def test_basic_cluster_minion_1(cluster_master_1, cluster_master_2, cluster_minion_1):
    cli = cluster_master_1.salt_cli(timeout=120)
    ret = cli.run("test.ping", minion_tgt="cluster-minion-1")
    assert ret.data is True


def test_basic_cluster_minion_1_from_master_2(
    cluster_master_1, cluster_master_2, cluster_minion_1
):
    cli = cluster_master_2.salt_cli(timeout=120)
    ret = cli.run("test.ping", minion_tgt="cluster-minion-1")
    assert ret.data is True


def test_basic_cluster_minion_1_from_master_3(
    cluster_master_1, cluster_master_2, cluster_master_3, cluster_minion_1
):
    cli = cluster_master_3.salt_cli(timeout=120)
    ret = cli.run("test.ping", minion_tgt="cluster-minion-1")
    assert ret.data is True



"""
Support for Linux LVM2
"""

import logging
import os.path

import salt.utils.path

# Set up logger
log = logging.getLogger(__name__)

# Define the module's virtual name
__virtualname__ = "lvm"


def __virtual__():
    """
    Only load the module if lvm is installed
    """
    if salt.utils.path.which("lvm"):
        return __virtualname__
    return (
        False,
        "The linux_lvm execution module cannot be loaded: the lvm binary is not in the"
        " path.",
    )


def version():
    """
    Return LVM version from lvm version

    CLI Example:

    .. code-block:: bash

        salt '*' lvm.version
    """
    cmd = "lvm version"
    out = __salt__["cmd.run"](cmd).splitlines()
    ret = out[0].split(": ")
    return ret[1].strip()


def fullversion():
    """
    Return all version info from lvm version

    CLI Example:

    .. code-block:: bash

        salt '*' lvm.fullversion
    """
    ret = {}
    cmd = "lvm version"
    out = __salt__["cmd.run"](cmd).splitlines()
    for line in out:
        comps = line.split(":")
        ret[comps[0].strip()] = comps[1].strip()
    return ret


def pvdisplay(pvname="", real=False, quiet=False):
    """
    Return information about the physical volume(s)

    pvname
        physical device name

    real
        dereference any symlinks and report the real device

        .. versionadded:: 2015.8.7

    quiet
        if the physical volume is not present, do not show any error

    CLI Examples:

    .. code-block:: bash

        salt '*' lvm.pvdisplay
        salt '*' lvm.pvdisplay /dev/md0
    """
    ret = {}
    cmd = ["pvdisplay", "-c"]
    if pvname:
        cmd.append(pvname)
    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False, ignore_retcode=quiet)

    if cmd_ret["retcode"] != 0:
        return {}

    out = cmd_ret["stdout"].splitlines()
    for line in out:
        if "is a new physical volume" not in line:
            comps = line.strip().split(":")
            if real:
                device = os.path.realpath(comps[0])
            else:
                device = comps[0]
            ret[device] = {
                "Physical Volume Device": comps[0],
                "Volume Group Name": comps[1],
                "Physical Volume Size (kB)": comps[2],
                "Internal Physical Volume Number": comps[3],
                "Physical Volume Status": comps[4],
                "Physical Volume (not) Allocatable": comps[5],
                "Current Logical Volumes Here": comps[6],
                "Physical Extent Size (kB)": comps[7],
                "Total Physical Extents": comps[8],
                "Free Physical Extents": comps[9],
                "Allocated Physical Extents": comps[10],
            }
            if real:
                ret[device]["Real Physical Volume Device"] = device
    return ret


def vgdisplay(vgname="", quiet=False):
    """
    Return information about the volume group(s)

    vgname
        volume group name

    quiet
        if the volume group is not present, do not show any error

    CLI Examples:

    .. code-block:: bash

        salt '*' lvm.vgdisplay
        salt '*' lvm.vgdisplay nova-volumes
    """
    ret = {}
    cmd = ["vgdisplay", "-c"]
    if vgname:
        cmd.append(vgname)
    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False, ignore_retcode=quiet)

    if cmd_ret["retcode"] != 0:
        return {}

    out = cmd_ret["stdout"].splitlines()
    for line in out:
        comps = line.strip().split(":")
        ret[comps[0]] = {
            "Volume Group Name": comps[0],
            "Volume Group Access": comps[1],
            "Volume Group Status": comps[2],
            "Internal Volume Group Number": comps[3],
            "Maximum Logical Volumes": comps[4],
            "Current Logical Volumes": comps[5],
            "Open Logical Volumes": comps[6],
            "Maximum Logical Volume Size": comps[7],
            "Maximum Physical Volumes": comps[8],
            "Current Physical Volumes": comps[9],
            "Actual Physical Volumes": comps[10],
            "Volume Group Size (kB)": comps[11],
            "Physical Extent Size (kB)": comps[12],
            "Total Physical Extents": comps[13],
            "Allocated Physical Extents": comps[14],
            "Free Physical Extents": comps[15],
            "UUID": comps[16],
        }
    return ret


def lvdisplay(lvname="", quiet=False):
    """
    Return information about the logical volume(s)

    lvname
        logical device name

    quiet
        if the logical volume is not present, do not show any error

    CLI Examples:

    .. code-block:: bash

        salt '*' lvm.lvdisplay
        salt '*' lvm.lvdisplay /dev/vg_myserver/root
    """
    ret = {}
    cmd = ["lvdisplay", "-c"]
    if lvname:
        cmd.append(lvname)
    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False, ignore_retcode=quiet)

    if cmd_ret["retcode"] != 0:
        return {}

    out = cmd_ret["stdout"].splitlines()
    for line in out:
        comps = line.strip().split(":")
        ret[comps[0]] = {
            "Logical Volume Name": comps[0],
            "Volume Group Name": comps[1],
            "Logical Volume Access": comps[2],
            "Logical Volume Status": comps[3],
            "Internal Logical Volume Number": comps[4],
            "Open Logical Volumes": comps[5],
            "Logical Volume Size": comps[6],
            "Current Logical Extents Associated": comps[7],
            "Allocated Logical Extents": comps[8],
            "Allocation Policy": comps[9],
            "Read Ahead Sectors": comps[10],
            "Major Device Number": comps[11],
            "Minor Device Number": comps[12],
        }
    return ret


def pvcreate(devices, override=True, force=True, **kwargs):
    """
    Set a physical device to be used as an LVM physical volume

    override
        Skip devices, if they are already LVM physical volumes

    CLI Examples:

    .. code-block:: bash

        salt mymachine lvm.pvcreate /dev/sdb1,/dev/sdb2
        salt mymachine lvm.pvcreate /dev/sdb1 dataalignmentoffset=7s
    """
    if not devices:
        return "Error: at least one device is required"
    if isinstance(devices, str):
        devices = devices.split(",")

    cmd = ["pvcreate"]

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    for device in devices:
        if not os.path.exists(device):
            return f"{device} does not exist"
        if not pvdisplay(device, quiet=True):
            cmd.append(device)
        elif not override:
            return f'Device "{device}" is already an LVM physical volume.'

    if not cmd[2:]:
        # All specified devices are already LVM volumes
        return True

    valid = (
        "metadatasize",
        "dataalignment",
        "dataalignmentoffset",
        "pvmetadatacopies",
        "metadatacopies",
        "metadataignore",
        "restorefile",
        "norestorefile",
        "labelsector",
        "setphysicalvolumesize",
    )
    no_parameter = "norestorefile"
    for var in kwargs:
        if kwargs[var] and var in valid:
            cmd.extend([f"--{var}", kwargs[var]])
        elif kwargs[var] and var in no_parameter:
            cmd.append(f"--{var}")

    out = __salt__["cmd.run_all"](cmd, python_shell=False)
    if out.get("retcode"):
        return out.get("stderr")

    # Verify pvcreate was successful
    for device in devices:
        if not pvdisplay(device):
            return f'Device "{device}" was not affected.'

    return True


def pvremove(devices, override=True, force=True):
    """
    Remove a physical device being used as an LVM physical volume

    override
        Skip devices, if they are already not used as LVM physical volumes

    CLI Examples:

    .. code-block:: bash

        salt mymachine lvm.pvremove /dev/sdb1,/dev/sdb2
    """
    if isinstance(devices, str):
        devices = devices.split(",")

    cmd = ["pvremove"]

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    for device in devices:
        if pvdisplay(device):
            cmd.append(device)
        elif not override:
            return f"{device} is not a physical volume"

    if not cmd[2:]:
        # Nothing to do
        return True

    out = __salt__["cmd.run_all"](cmd, python_shell=False)
    if out.get("retcode"):
        return out.get("stderr")

    # Verify pvremove was successful
    for device in devices:
        if pvdisplay(device, quiet=True):
            return f'Device "{device}" was not affected.'

    return True


def vgcreate(vgname, devices, force=False, **kwargs):
    """
    Create an LVM volume group

    CLI Examples:

    .. code-block:: bash

        salt mymachine lvm.vgcreate my_vg /dev/sdb1,/dev/sdb2
        salt mymachine lvm.vgcreate my_vg /dev/sdb1 clustered=y
    """
    if not vgname or not devices:
        return "Error: vgname and device(s) are both required"
    if isinstance(devices, str):
        devices = devices.split(",")

    cmd = ["vgcreate", vgname]
    for device in devices:
        cmd.append(device)

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    valid = (
        "addtag",
        "alloc",
        "autobackup",
        "clustered",
        "maxlogicalvolumes",
        "maxphysicalvolumes",
        "metadatatype",
        "vgmetadatacopies",
        "metadatacopies",
        "physicalextentsize",
        "zero",
    )
    for var in kwargs:
        if kwargs[var] and var in valid:
            cmd.append(f"--{var}")
            cmd.append(kwargs[var])

    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False)
    if cmd_ret.get("retcode"):
        out = cmd_ret.get("stderr").strip()
    else:
        out = f'Volume group "{vgname}" successfully created'

    vgdata = vgdisplay(vgname)
    vgdata["Output from vgcreate"] = out
    return vgdata


def vgextend(vgname, devices, force=False):
    """
    Add physical volumes to an LVM volume group

    CLI Examples:

    .. code-block:: bash

        salt mymachine lvm.vgextend my_vg /dev/sdb1,/dev/sdb2
        salt mymachine lvm.vgextend my_vg /dev/sdb1
    """
    if not vgname or not devices:
        return "Error: vgname and device(s) are both required"
    if isinstance(devices, str):
        devices = devices.split(",")

    cmd = ["vgextend", vgname]

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    for device in devices:
        cmd.append(device)

    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False)
    if cmd_ret.get("retcode"):
        out = cmd_ret.get("stderr").strip()
    else:
        out = f'Volume group "{vgname}" successfully extended'

    vgdata = {"Output from vgextend": out}
    return vgdata


def lvcreate(
    lvname,
    vgname,
    size=None,
    extents=None,
    snapshot=None,
    pv=None,
    thinvolume=False,
    thinpool=False,
    force=False,
    **kwargs,
):
    """
    Create a new logical volume, with option for which physical volume to be used

    CLI Examples:

    .. code-block:: bash

        salt '*' lvm.lvcreate new_volume_name     vg_name size=10G
        salt '*' lvm.lvcreate new_volume_name     vg_name extents=100 pv=/dev/sdb
        salt '*' lvm.lvcreate new_snapshot        vg_name snapshot=volume_name size=3G

    .. versionadded:: 0.12.0

    Support for thin pools and thin volumes

    CLI Examples:

    .. code-block:: bash

        salt '*' lvm.lvcreate new_thinpool_name   vg_name               size=20G thinpool=True
        salt '*' lvm.lvcreate new_thinvolume_name vg_name/thinpool_name size=10G thinvolume=True

    """
    if size and extents:
        return "Error: Please specify only one of size or extents"
    if thinvolume and thinpool:
        return "Error: Please set only one of thinvolume or thinpool to True"

    valid = (
        "activate",
        "chunksize",
        "contiguous",
        "discards",
        "stripes",
        "stripesize",
        "minor",
        "persistent",
        "mirrors",
        "nosync",
        "noudevsync",
        "monitor",
        "ignoremonitoring",
        "permission",
        "poolmetadatasize",
        "readahead",
        "regionsize",
        "type",
        "virtualsize",
        "zero",
    )
    no_parameter = (
        "nosync",
        "noudevsync",
        "ignoremonitoring",
        "thin",
    )

    extra_arguments = []
    if kwargs:
        for k, v in kwargs.items():
            if k in no_parameter:
                extra_arguments.append(f"--{k}")
            elif k in valid:
                extra_arguments.extend([f"--{k}", f"{v}"])

    cmd = [salt.utils.path.which("lvcreate")]

    if thinvolume:
        cmd.extend(["--thin", "-n", lvname])
    elif thinpool:
        cmd.extend(["--thinpool", lvname])
    else:
        cmd.extend(["-n", lvname])

    if snapshot:
        cmd.extend(["-s", f"{vgname}/{snapshot}"])
    else:
        cmd.append(vgname)

    if size and thinvolume:
        cmd.extend(["-V", f"{size}"])
    elif extents and thinvolume:
        return "Error: Thin volume size cannot be specified as extents"
    elif size:
        cmd.extend(["-L", f"{size}"])
    elif extents:
        cmd.extend(["-l", f"{extents}"])
    else:
        return "Error: Either size or extents must be specified"

    if pv:
        cmd.append(pv)
    if extra_arguments:
        cmd.extend(extra_arguments)

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False)
    if cmd_ret.get("retcode"):
        out = cmd_ret.get("stderr").strip()
    else:
        out = f'Logical volume "{lvname}" created.'

    lvdev = f"/dev/{vgname}/{lvname}"
    lvdata = lvdisplay(lvdev)
    lvdata["Output from lvcreate"] = out
    return lvdata


def vgremove(vgname, force=True):
    """
    Remove an LVM volume group

    CLI Examples:

    .. code-block:: bash

        salt mymachine lvm.vgremove vgname
        salt mymachine lvm.vgremove vgname force=True
    """
    cmd = ["vgremove", vgname]

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False)
    if cmd_ret.get("retcode"):
        out = cmd_ret.get("stderr").strip()
    else:
        out = f'Volume group "{vgname}" successfully removed'
    return out


def lvremove(lvname, vgname, force=True):
    """
    Remove a given existing logical volume from a named existing volume group

    CLI Example:

    .. code-block:: bash

        salt '*' lvm.lvremove lvname vgname force=True
    """
    cmd = ["lvremove", f"{vgname}/{lvname}"]

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False)
    if cmd_ret.get("retcode"):
        out = cmd_ret.get("stderr").strip()
    else:
        out = f'Logical volume "{lvname}" successfully removed'

    return out


def lvresize(size=None, lvpath=None, extents=None, force=False, resizefs=False):
    """
    Resize a logical volume to specific size.

    CLI Examples:

    .. code-block:: bash


        salt '*' lvm.lvresize +12M /dev/mapper/vg1-test
        salt '*' lvm.lvresize lvpath=/dev/mapper/vg1-test extents=+100%FREE

    """
    if size and extents:
        log.error("Error: Please specify only one of size or extents")
        return {}

    cmd = ["lvresize"]

    if force:
        cmd.append("--force")
    else:
        cmd.append("-qq")

    if resizefs:
        cmd.append("--resizefs")

    if size:
        cmd.extend(["-L", f"{size}"])
    elif extents:
        cmd.extend(["-l", f"{extents}"])
    else:
        log.error("Error: Either size or extents must be specified")
        return {}

    cmd.append(lvpath)

    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False)
    if cmd_ret.get("retcode"):
        out = cmd_ret.get("stderr").strip()
    else:
        out = f'Logical volume "{lvpath}" successfully resized.'

    return {"Output from lvresize": out}


def lvextend(size=None, lvpath=None, extents=None, force=False, resizefs=False):
    """
    Increase a logical volume to specific size.

    CLI Examples:

    .. code-block:: bash


        salt '*' lvm.lvextend +12M /dev/mapper/vg1-test
        salt '*' lvm.lvextend lvpath=/dev/mapper/vg1-test extents=+100%FREE

    """
    if size and extents:
        log.error("Error: Please specify only one of size or extents")
        return {}

    cmd = ["lvextend"]

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    if resizefs:
        cmd.append("--resizefs")

    if size:
        cmd.extend(["-L", f"{size}"])
    elif extents:
        cmd.extend(["-l", f"{extents}"])
    else:
        log.error("Error: Either size or extents must be specified")
        return {}

    cmd.append(lvpath)

    cmd_ret = __salt__["cmd.run_all"](cmd, python_shell=False)
    if cmd_ret.get("retcode"):
        out = cmd_ret.get("stderr").strip()
    else:
        out = f'Logical volume "{lvpath}" successfully extended.'

    return {"Output from lvextend": out}


def pvresize(devices, override=True, force=True):
    """
    Resize a LVM physical volume to the physical device size

    override
        Skip devices, if they are already not used as LVM physical volumes

    CLI Examples:

    .. code-block:: bash

        salt mymachine lvm.pvresize /dev/sdb1,/dev/sdb2
    """
    if isinstance(devices, str):
        devices = devices.split(",")

    cmd = ["pvresize"]

    if force:
        cmd.append("--yes")
    else:
        cmd.append("-qq")

    for device in devices:
        if pvdisplay(device):
            cmd.append(device)
        elif not override:
            return f"{device} is not a physical volume"

    if not cmd[2:]:
        # Nothing to do
        return True

    out = __salt__["cmd.run_all"](cmd, python_shell=False)
    if out.get("retcode"):
        return out.get("stderr")

    return True



"""
Simple Smoke Tests for Connected SSH minions
"""

import subprocess

import packaging.version
import pytest
from saltfactories.utils.functional import StateResult

import salt.utils.platform
import salt.utils.versions

pytestmark = [
    pytest.mark.slow_test,
    pytest.mark.skip_on_windows(reason="salt-ssh not available on Windows"),
]


def _check_systemctl():
    if not hasattr(_check_systemctl, "memo"):
        if not salt.utils.platform.is_linux():
            _check_systemctl.memo = False
        else:
            proc = subprocess.run(["systemctl"], capture_output=True, check=False)
            _check_systemctl.memo = (
                b"Failed to get D-Bus connection: No such file or directory"
                in proc.stderr
            )
    return _check_systemctl.memo


def _check_python():
    try:
        proc = subprocess.run(
            ["/usr/bin/python3", "--version"], capture_output=True, check=False
        )
    except FileNotFoundError:
        return True
    return packaging.version.Version(
        proc.stdout.decode().strip().split()[1]
    ) <= packaging.version.Version("3.10")


@pytest.mark.skip_if_not_root
@pytest.mark.skipif(_check_systemctl(), reason="systemctl degraded")
@pytest.mark.skipif(_check_python(), reason="System python less than 3.10")
def test_service(salt_ssh_cli, grains):
    service = "cron"
    os_family = grains["os_family"]
    os_release = grains["osrelease"]
    if os_family == "RedHat":
        service = "crond"
    elif os_family == "Arch":
        service = "sshd"
    elif os_family == "MacOS":
        service = "org.ntp.ntpd"
        if int(os_release.split(".")[1]) >= 13:
            service = "com.apple.AirPlayXPCHelper"
    ret = salt_ssh_cli.run("service.enable", service)
    assert ret.returncode == 0
    ret = salt_ssh_cli.run("service.get_all")
    assert ret.returncode == 0
    assert service in ret.data
    ret = salt_ssh_cli.run("service.stop", service)
    assert ret.returncode == 0
    ret = salt_ssh_cli.run("service.status", service)
    assert ret.returncode == 0
    assert not ret.data
    ret = salt_ssh_cli.run("service.start", service)
    assert ret.returncode == 0
    ret = salt_ssh_cli.run("service.status", service)
    assert ret.returncode == 0
    assert ret.data


@pytest.fixture
def _state_tree(salt_master, tmp_path):
    top_sls = """
    base:
      '*':
        - core
    """
    core_state = """
    {}/testfile:
      file:
        - managed
        - source: salt://testfile
        - makedirs: true
    """.format(
        tmp_path
    )
    with salt_master.state_tree.base.temp_file(
        "top.sls", top_sls
    ), salt_master.state_tree.base.temp_file("core.sls", core_state):
        yield


@pytest.fixture
def custom_wrapper(salt_run_cli, base_env_state_tree_root_dir):
    module_contents = r"""\
def __virtual__():
    return "grains_custom"

def items():
    return __grains__.value()
    """
    module_dir = base_env_state_tree_root_dir / "_wrapper"
    module_tempfile = pytest.helpers.temp_file(
        "grains_custom.py", module_contents, module_dir
    )
    try:
        with module_tempfile:
            ret = salt_run_cli.run("saltutil.sync_wrapper")
            assert ret.returncode == 0
            assert "wrapper.grains_custom" in ret.data
            yield
    finally:
        ret = salt_run_cli.run("saltutil.sync_wrapper")
        assert ret.returncode == 0


@pytest.mark.usefixtures("_state_tree")
def test_state_apply(salt_ssh_cli):
    ret = salt_ssh_cli.run("state.apply", "core")
    assert ret.returncode == 0
    state_result = StateResult(ret.data)
    assert state_result.result is True


@pytest.mark.usefixtures("_state_tree")
def test_state_highstate(salt_ssh_cli):
    ret = salt_ssh_cli.run("state.highstate")
    assert ret.returncode == 0
    state_result = StateResult(ret.data)
    assert state_result.result is True


@pytest.mark.usefixtures("custom_wrapper")
def test_custom_wrapper(salt_ssh_cli):
    ret = salt_ssh_cli.run(
        "grains_custom.items",
    )
    assert ret.returncode == 0
    assert ret.data
    assert "id" in ret.data
    assert ret.data["id"] in ("localhost", "127.0.0.1")
